# -*- coding: Latin-1 -*-
#
# SMM Share and Money Management
# Copyright (c) 2007-2010 Carsten Grohmann
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 2 of the
# License.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

## \file gui/__init__.py
#
# \brief Main file of the SMM gui
# 
# \author Carsten Grohmann <mail (at) carstengrohmann (dot) de>
# 
# \par License:
# GPL version 2.0

## \page channels Publisher Channels
#
# <table>
#   <tr>
#     <td>Channel</td>
#     <td>\code .add \endcode</td>
#     <td>\code .change \endcode</td>
#     <td>\code .remove \endcode</td>
#     <td>Transfered datum</td>
#   </tr>
#   <tr>
#     <td>\code smm \endcode</td>
#     <td>&nbsp;</td>
#     <td>&nbsp;</td>
#     <td>&nbsp;</td>
#     <td>Parent channel, subscribe it to get all messages</td>
#   </tr>
#   <tr>
#     <td>\code smm.currency \endcode</td>
#     <td>X</td>
#     <td>X</td>
#     <td>&nbsp;</td>
#     <td>smm.objects.Currency.id</td>
#   </tr>
#   <tr>
#     <td>\code smm.exchange \endcode</td>
#     <td>X</td>
#     <td>X</td>
#     <td>&nbsp;</td>
#     <td>smm.objects.Exchange.id</td>
#   </tr>
#   <tr>
#     <td>\code smm.quote \endcode</td>
#     <td>&nbsp;</td>
#     <td>&nbsp;</td>
#     <td>&nbsp;</td>
#     <td>smm.objects.DailyQuote.id</td>
#   </tr>
#   <tr>
#     <td>\code smm.position \endcode</td>
#     <td>X</td>
#     <td>X</td>
#     <td>X</td>
#     <td>smm.objects.Position.id</td>
#   </tr>
#   <tr>
#     <td>\code smm.share \endcode</td>
#     <td>X</td>
#     <td>X</td>
#     <td>&nbsp;</td>
#     <td>smm.objects.Share.id</td>
#   </tr>
#   <tr>
#     <td>\code smm.transaction \endcode</td>
#     <td>X</td>
#     <td>X</td>
#     <td>&nbsp;</td>
#     <td>smm.objects.Transaction.id</td>
#   </tr>
#   <tr>
#     <td>\code smm.watch \endcode</td>
#     <td>X</td>
#     <td>X</td>
#     <td>X</td>
#     <td>smm.objects.Watch.id</td>
#   </tr>
#   <tr>
#     <td>\code smm.position \endcode</td>
#     <td>X</td>
#     <td>X</td>
#     <td>&nbsp;</td>
#     <td>smm.objects.Position.id</td>
#   </tr>
# </table>

# import other general python modules
import atexit
import datetime
import re
import sys
import StringIO

# import gettext and initiate dummy Unicode instance
import gettext
gettext.NullTranslations().install(True)

# initialise own logging extensions
import logging
import log
log.init()
atexit.register(log.deinit)

## Message about installed Python versions
py_version_installed = _(u'Zur Zeit ist Python in der Version %s installiert')

## Too old wxPython error message
wx_version_err = \
    _(u'Dieses Programm benoetigt mindestens wxPython Version %s')

## wxPython isn't installed error message
wx_notinstalled_err = _(u'wxPython ist nicht installiert.')

## Message about installed wxPython versions
wx_version_installed = _(u'Zur Zeit ist wxPython in der Version %s installiert')

## Minimum wxPython version
wx_min_version = '2.8'

# check for minimum python version
if sys.hexversion < 0x020500f0:
    logging.error(
        _(u'Dieses Programm benoetigt mindestens Python Version 2.5')
        )
    logging.error(py_version_installed, re.sub('\n', '', sys.version))
    sys.exit(1)

# check for recommended python version
if sys.hexversion < 0x020502f0:
    logging.warning(
        _(u'Es ist empfohlen Python mindestens in der Version 2.5.2 einzusetzen.')
        )
    logging.warning(py_version_installed, re.sub('\n', '', sys.version))
    
# import proper wx-module using wxversion
if not hasattr(sys, "frozen") and 'wx' not in sys.modules:
    try:
        import wxversion
        if wxversion.checkInstalled(wx_min_version):
            wxversion.select(wx_min_version)
        else:
            logging.error(wx_version_err, wx_min_version)
            logging.error(
                wx_version_installed,
                _(u', ').join(wxversion.getInstalled())
                )
            sys.exit(1)
    except ImportError:
        pass

try:
    import wx
    if hasattr(wx, 'version'):
        wx_cur_version = wx.version()
    else:
        wx_cur_version = wx.VERSION_STRING
    if not wx_cur_version.startswith(wx_min_version):
        logging.error(wx_version_err, wx_min_version)
        logging.error(wx_version_installed, wx_cur_version)
        sys.exit(1)
except ImportError:
    logging.error(wx_version_err, wx_min_version)
    logging.error(wx_notinstalled_err)
    sys.exit(1)

# Default encoding of all non-unicode strings in the source files
wx.SetDefaultPyEncoding("Latin-1")

# import main lib
import smm

# import gui modules
import config
import dialogs
import helper
import reports
from panel import *

## Application main class
#
#  Initialize the application and main window
class SMMGui(wx.App):

    ## Contructor
    #
    #  \param self      The object pointer
    #  \param redirect  Redirect sys.stdout and sys.stderr to a separate
    #                   window
    #  \param filename  Redirect to a file instead of a window
    def __init__(self, redirect=False, filename=None):

        # initialise configuration
        config.init()

        # Restart logging using configured log level and file encoding
        log.init(
            encoding=config.get_item('logging/encoding'),
            level=config.get_item('logging/level'),
            )

        # initialise SMM stuff
        smm.db_uri = u'sqlite:%s' % config.get_item('database/file')
        if smm.has_database():
            smm.init()

        # call parent constructor
        wx.App.__init__(self, redirect, filename)


    ## wxWindows calls this method to initialize the application
    def OnInit(self):

        # Create an instance of our customized Frame class
        frame = MainWindow(
            parent=None,
            id=wx.ID_ANY,
            title=_(u"Share and Money Management"),
            )

        frame.Show(True)

        # Tell wxWindows that this is our main window
        self.SetTopWindow(frame)

        # Return a success flag
        return True

    ## Do some cleanup actions
    def OnExit(self):
        # finalise SMM stuff
        if  smm.database.store:
            smm.deinit()

        # store configuration
        config.save()


## Main window of our application
#
#  The application is an MDI application to provide the user more flexibility
#  in selection and positioning of the displayed data.
#
#  The content of an MDI child is a simple \c wx.Frame. That unifies the
#  handling of the MDI childs.
class MainWindow(wx.MDIParentFrame):

    ## \var nativeMDI
    #  Indicates native MDI support.
    nativeMDI = False

    ## \var menubar
    #  \brief Main menubar with all submenus
    #  \see filemenu and helpmenu 

    ## \var views_open
    #  List of all open views
    views_open = []

    ## \var filemenu
    #  All menue entries grouped to the "File" menu

    ## \var helpmenu
    #  All menue entries grouped to the "Help" menu

    ## \var managemenu
    #  All menue entries grouped to the "Manage" menu

    ## \var reportmenu
    #  All menue entries grouped to the "Reports" menu
    
    ## \var viewmenu
    #  All menue entries grouped to the "View" menu
    
    ## \var ViewSummary
    #  Reference to the "Summary" MDI client panel

    ## \var ViewCurrency
    #  Reference to the "Currency" MDI client panel

    ## \var ViewExchange
    #  Reference to the "Exchange" MDI client panel

    ## \var ViewShare
    #  Reference to the "Share" MDI client panel

    ## \var ViewTransaction
    #  Reference to the "Transaction" MDI client panel

    ## \var ViewIndex
    #  Reference to the "Indices" MDI client panel

    ## \var ViewPositionList
    #  Reference to the "OpenPositionList" MDI client panel

    ## \var ViewPositionDetails
    #  Reference to the "OpenPositionDetails" MDI client panel

    ID_FILEMENU_PREFERENCES       = 101

    ID_MANAGEMENU_FETCHQUOTES     = 201
    ID_MANAGEMENU_ADDINDEX        = 202
    ID_MANAGEMENU_ADDSHARE        = 203
    ID_MANAGEMENU_ADDTRANSACTION  = 204

    ID_REPORTMENU_POSITIONREPORT  = 301
    ID_REPORTMENU_PORTFOLIOREPORT = 302

    ID_VIEWMENU_SUMMARY           = 401
    ID_VIEWMENU_CURRENCY          = 402
    ID_VIEWMENU_EXCHANGE          = 403
    ID_VIEWMENU_SHARE             = 404
    ID_VIEWMENU_TRANSACTION       = 405
    ID_VIEWMENU_INDEX             = 406
    ID_VIEWMENU_POSITIONLIST      = 407
    ID_VIEWMENU_POSITIONDETAILS   = 408
    ID_VIEWMENU_WATCHLIST         = 409

    ## Mapping between view name and menu id
    attr2menu = {
        'ViewSummary':         ID_VIEWMENU_SUMMARY,
        'ViewCurrency':        ID_VIEWMENU_CURRENCY,
        'ViewExchange':        ID_VIEWMENU_EXCHANGE,
        'ViewShare':           ID_VIEWMENU_SHARE,
        'ViewTransaction':     ID_VIEWMENU_TRANSACTION,
        'ViewIndex':           ID_VIEWMENU_INDEX,
        'ViewPositionList':    ID_VIEWMENU_POSITIONLIST,
        'ViewPositionDetails': ID_VIEWMENU_POSITIONDETAILS,
        'ViewWatchList':       ID_VIEWMENU_WATCHLIST,
        }

    ## Constructor
    def __init__(self, parent, id, title):
        wx.MDIParentFrame.__init__(self, parent, id, title, size=(400, 400),
                style=wx.DEFAULT_FRAME_STYLE|wx.NO_FULL_REPAINT_ON_RESIZE)

        # initialise internal variables
        self.filemenu            = None
        self.helpmenu            = None
        self.managemenu          = None
        self.reportmenu          = None
        self.viewmenu            = None
        self.menubar             = None
        self.ViewSummary         = None
        self.ViewCurrency        = None
        self.ViewExchange        = None
        self.ViewShare           = None
        self.ViewTransaction     = None
        self.ViewIndex           = None
        self.ViewPositionList    = None
        self.ViewPositionDetails = None
        self.ViewWatchList       = None

        # check for native MDI support
        if 'wxMSW' in wx.PlatformInfo or 'wxMAC' in wx.PlatformInfo:
            self.nativeMDI = True

        # create a statusbar in the bottom of the window
        self.CreateStatusBar()

        # create all menus
        self.createMenus()

        # restore GUI settings
        self.restoreGUIConfig()

        # disable menus if SMM not available
        if not smm.database.store:
            # disable SMM menu elements
            self.menubar.EnableTop(
                self.menubar.FindMenu(_(u"&Verwalten")),
                False
                )
            self.menubar.EnableTop(
                self.menubar.FindMenu(_(u"&Berichte")),
                False
                )
            self.menubar.EnableTop(
                self.menubar.FindMenu(_(u"&Ansichten")),
                False
                )
        else:
            # restore SMM related GUI settings
            self.restoreSMMConfig()

        # add event handler to save configuration on exit
        self.Bind(wx.EVT_CLOSE, self.OnClose)

        # show warning about missing database and open preferences dialog
        if not smm.database.store:
            wx.MessageBox(
                _(u'Die Datenbank ist nicht verfgbar.\n' \
                  u'Bitte whlen Sie eine existierende Datenbank oder' \
                  u'legen Sie eine neue Datenbank an.'
                ),
                caption=_(u'Hinweis'),
                style=wx.OK|wx.ICON_EXCLAMATION,
                )
            self.OnPreferences(None)


    ## Create all menus
    def createMenus(self):

        # create single menu parts
        self.createFileMenu()
        self.createManageMenu()
        self.createHelpMenu()
        self.createReportMenu()
        self.createViewMenu()

        # add all menus to the menubar
        self.menubar = wx.MenuBar()
        self.menubar.Append(self.filemenu,   _(u"&Datei"))
        self.menubar.Append(self.managemenu, _(u"&Verwalten"))
        self.menubar.Append(self.reportmenu, _(u"&Berichte"))
        self.menubar.Append(self.viewmenu,   _(u"&Ansichten"))
        self.menubar.Append(self.helpmenu,   _(u"&Hilfe"))

        # Adding the MenuBar to the Frame content
        self.SetMenuBar(self.menubar)

    ## Create a single menu item
    #
    #  \param self  The object pointer
    #  \param menu  Menu to append
    #  \param mid   ID of the entry
    #  \param item  Menu text
    #  \param htext Help text
    #  \param func  Function to call if the menu event occure
    #  \param kind  Kind of the menu entry
    def createMenuEntry(self, menu, mid, item, htext, func, kind=wx.ITEM_NORMAL):
        menu.Append(mid, item, htext, kind)
        self.Bind(wx.EVT_MENU, func, id=mid)

    ## Create "File" menu
    def createFileMenu(self):
        self.filemenu = wx.Menu()
        self.createMenuEntry(
            self.filemenu,
            self.ID_FILEMENU_PREFERENCES,
            _(u"SMM einrichten"),
            _(u"Verschiedene Konfigurationsoptionen festlegen"),
            self.OnPreferences,
            )
        self.filemenu.AppendSeparator()
        self.createMenuEntry(
            self.filemenu,
            wx.ID_EXIT,
            _(u"Be&enden\tCtrl+Q"),
            _(u"Beenden"),
            self.OnFileExit
            )

    ## Create "Manage" menu
    def createManageMenu(self):
        self.managemenu = wx.Menu()
        self.createMenuEntry(
            self.managemenu,
            self.ID_MANAGEMENU_FETCHQUOTES,
            _(u"Kursdaten abfragen"),
            _(u"Kursdaten aus verschiedenen Quellen abfragen"),
            self.OnFetchQuotes
            )
        self.managemenu.AppendSeparator()
        self.createMenuEntry(
            self.managemenu,
            self.ID_MANAGEMENU_ADDINDEX,
            _(u"Neuen Index anlegen"),
            _(u"Einen neuen Index anlegen"),
            self.OnNewIndex
            )
        self.createMenuEntry(
            self.managemenu,
            self.ID_MANAGEMENU_ADDTRANSACTION,
            _(u"Neue Transaktion anlegen"),
            _(u"Mit einem Wertpapier handeln"),
            self.OnNewTransaction
            )
        self.createMenuEntry(
            self.managemenu,
            self.ID_MANAGEMENU_ADDSHARE,
            _(u"Neues Wertpapier anlegen"),
            _(u"Ein neues Wertpapier anlegen"),
            self.OnNewShare
            )

    ## Create "Reports" menu
    def createReportMenu(self):
        self.reportmenu = wx.Menu()
        self.createMenuEntry(
            self.reportmenu,
            self.ID_REPORTMENU_POSITIONREPORT,
            _(u"&Positionsbericht"),
            _(u"Bericht ber eine einzele Position"),
            self.OnPositionReport
            )
        self.createMenuEntry(
            self.reportmenu,
            self.ID_REPORTMENU_PORTFOLIOREPORT,
            _(u"&Portfoliobericht"),
            _(u"Bericht ber Entwicklung des Portfolio fr einen Zeitraum"),
            self.OnPortfolioReport
            )

    ## Create "View" menu
    def createViewMenu(self):
        self.viewmenu = wx.Menu()
        self.createMenuEntry(
            self.viewmenu,
            self.ID_VIEWMENU_SUMMARY,
            _(u"&bersicht"),
            _(u"Zeigt eine Zusammenfassung des Portfolios"),
            self.OnCreateSummary,
            wx.ITEM_CHECK
            )

        self.createMenuEntry(
            self.viewmenu,
            self.ID_VIEWMENU_CURRENCY,
            _(u"&Whrungen"),
            _(u"Zeigt alle bekannten Whrungen"),
            self.OnCreateCurrency,
            wx.ITEM_CHECK
            )

        self.createMenuEntry(
            self.viewmenu,
            self.ID_VIEWMENU_EXCHANGE,
            _(u"&Brsen"),
            _(u"Zeigt alle bekannten Brsenpltze"),
            self.OnCreateExchange,
            wx.ITEM_CHECK
            )

        self.createMenuEntry(
            self.viewmenu,
            self.ID_VIEWMENU_SHARE,
            _(u"Wertpapiere"),
            _(u"Zeigt alle bekannten Wertpapiere"),
            self.OnCreateShare,
            wx.ITEM_CHECK
            )

        self.createMenuEntry(
            self.viewmenu,
            self.ID_VIEWMENU_TRANSACTION,
            _(u"Transaktionen"),
            _(u"Zeigt alle Transaktionen"),
            self.OnCreateTransaction,
            wx.ITEM_CHECK
            )

        self.createMenuEntry(
            self.viewmenu,
            self.ID_VIEWMENU_INDEX,
            _(u"Indices"),
            _(u"Zeigt alle Indices"),
            self.OnCreateIndex,
            wx.ITEM_CHECK
            )

        self.createMenuEntry(
            self.viewmenu,
            self.ID_VIEWMENU_POSITIONLIST,
            _(u"Offene Positionen - Liste"),
            _(u"Zeigt alle offenen Positionen in einer Listenform"),
            self.OnCreatePositionList,
            wx.ITEM_CHECK
            )

        self.createMenuEntry(
            self.viewmenu,
            self.ID_VIEWMENU_POSITIONDETAILS,
            _(u"Offene Positionen - Details"),
            _(u"Zeigt alle offenen Positionen in einer Detailbersicht"),
            self.OnCreatePositionDetails,
            wx.ITEM_CHECK
            )

        self.createMenuEntry(
            self.viewmenu,
            self.ID_VIEWMENU_WATCHLIST,
            _(u"Beobachtungsliste"),
            _(u"Zeigt alle Wertpapiere, die zur Zeit beobachtet werden"),
            self.OnCreateWatchList,
            wx.ITEM_CHECK
            )

    ## Create "Help" menu
    def createHelpMenu(self):
        self.helpmenu = wx.Menu()
        self.createMenuEntry(
            self.helpmenu,
            wx.ID_ABOUT,
            _(u"&ber"),
            _(u"Zeigt Informationen ber diese Software"),
            self.OnAbout
            )

    ## Open about dialog
    #
    #  \see gui.dialogs.showAboutDialog()
    def OnAbout(self, event):
        dialogs.showAboutDialog(self, smm.version, smm.storm_version)

    ## Finish this application
    def OnFileExit(self, event):
        # destroy the main widget and all child widgets
        self.Close()

    ## Open fetch quotes dialog
    #
    #  \see gui.dialogs.DlgFetchQuotes
    def OnFetchQuotes(self, event):
        dialog = dialogs.DlgFetchQuotes(self)
        dialog.Centre(wx.BOTH)
        dialog.ShowModal()
        dialog.Destroy()

    ## Open new index dialog
    #
    #  \see gui.dialogs.DlgShare and
    #       gui.dialogs.DlgShare.setFixedShareKind
    def OnNewIndex(self, event):
        dialog = dialogs.DlgShare(self)
        dialog.setFixedShareKind('index')
        dialog.Centre(wx.BOTH)
        dialog.ShowModal()
        dialog.Destroy()

    ## Open new share dialog
    #
    #  \see gui.dialogs.DlgShare
    def OnNewShare(self, event):
        dialog = dialogs.DlgShare(self)
        dialog.Centre(wx.BOTH)
        dialog.ShowModal()
        dialog.Destroy()

    ## Open new transaction dialog
    #
    #  \see gui.dialogs.DlgTrade
    def OnNewTransaction(self, event):
        dialog = dialogs.DlgTrade(self)
        dialog.Centre(wx.BOTH)
        dialog.ShowModal()
        dialog.Destroy()

    ## Open a new dialog to show a position report
    #
    #  \see gui.dialogs.DlgHtml
    def OnPositionReport(self, event):        
        if not smm.Position.getCount():
            wx.MessageBox(
                _(u'Der Report kann nicht angezeigt werden, da es keine Positionen gibt.'),
                _(u"Hinweis")
                )
            return
  
        # Select position to report first
        dlg_pos = dialogs.DlgPositionReport(self)
        dlg_pos.Centre(wx.BOTH)
        result = dlg_pos.ShowModal()
        if result != wx.ID_OK:
            dlg_pos.Destroy()
            return

        pos = smm.Position.get(dlg_pos.GetSelection())
        dlg_pos.Destroy()

        # Create and show report
        dlg = dialogs.DlgShowPositionReport(self)
        dlg.ShowModal(pos)

    ## Open a new dialog to show a depot report
    #
    #  \see gui.dialogs.DlgHtml
    def OnPortfolioReport(self, event):        
        if not smm.Position.getCount():
            wx.MessageBox(
                _(u'Der Report kann nicht angezeigt werden, da es keine Positionen gibt.'),
                _(u"Hinweis")
                )

            return

        start = end = None
        dlg = dialogs.DlgPortfolioReport(self)
        dlg.Centre(wx.BOTH)
        if dlg.ShowModal() == wx.ID_OK:
            start = dlg.getStartDate()
            end = dlg.getEndDate()
        dlg.Destroy()
        if not (start and end):
            return

        # Create and show report
        dlg_report = dialogs.DlgHtml(self)
        dlg_report.SetTitle(
            _(u'Portfoliobersicht')
            )
        buf = StringIO.StringIO()
        reports.createPortfolioReport(start, end, buf)
        dlg_report.setHtml(buf.getvalue())
        dlg_report.Centre(wx.BOTH)
        dlg_report.ShowModal()
        dlg_report.Destroy()

    ## Open preferences dialog
    #
    #  \see gui.dialogs.DlgPreferences
    def OnPreferences(self, event):
        dlg = dialogs.DlgPreferences(
            self,
            db_file=config.get_item('database/file'),
            log_enc=config.get_item('logging/encoding').lower(),
            log_level=config.get_item('logging/level').lower(),
            )
        dlg.Centre(wx.BOTH)

        has_changed = False

        # return on cancel
        if dlg.ShowModal() != wx.ID_OK:
            dlg.Destroy()
            return

        # set changes changes
        if config.get_item('database/file') != dlg.db_file:
            config.set_item('database/file', dlg.db_file)
            has_changed = True
        if config.get_item('logging/encoding') != dlg.log_enc:
            config.set_item('logging/encoding', dlg.log_enc)
            has_changed = True
        if config.get_item('logging/level') != dlg.log_level:
            config.set_item('logging/level', dlg.log_level)
            has_changed = True

        # free dialog resources
        dlg.Destroy()

        # show warning on changes
        if has_changed:
            wx.MessageBox(
                _(u'Die nderungen an den Einstellungen werden erst nach ' \
                   'einem Neustart der Anwendung aktiv.'),
                caption=_(u'Hinweis'),
                style=wx.OK|wx.ICON_INFORMATION,
                )



    #==============================
    # Functions to create new views
    #==============================


    ## Creates and closes MDI child windows.
    #
    #  The function toggles the state of a MDI child between created and
    #  closed. A created child has a valid reference.
    #
    #  New childs are setted up by creating a \c wx.MDIChildFrame and
    #  placing the content in form of a \c wx.Frame in it.
    #
    #  The menu item will be disables for native MDI platforms and a
    #  event handler for the close event will be installed to reenable
    #  the menu item.
    #
    #  The state of the menu entry is set depending of the platform.
    #
    #  \param self        The object pointer
    #  \param panelClass  Class that provides the content of the window.
    #  \param attr        Name of the attribute to store the reference
    #                     to the MDI child.
    #  \param closeEvent  Handler for MDI childs close event
    #  \param title       Title of the MDI child window
    def handleMDIChild(self, panelClass, attr, closeEvent, title):
        if self.viewmenu.IsChecked(self.attr2menu[attr]):
            child = wx.MDIChildFrame(self, wx.ID_ANY, title)
            panel = panelClass(child, wx.ID_ANY)

            # bind panels handler for close events too
            if hasattr(panel, 'OnClose'):
                child.Bind(wx.EVT_CLOSE, panel.OnClose)

            child.Bind(wx.EVT_CLOSE, closeEvent)

            child.Show(True)

            # store view name in MDI child to restore the last active
            # view after a program restart
            child.viewName = attr

            setattr(self, attr, child)
            self.views_open.append(attr)

        else:
            child = getattr(self, attr)
            if not child:
                logging.warning(u'Ignoring unknown child "%s".', attr)
                return
            child.Hide()
            child.Close()


    ## Show a summary about the current deposit.
    #
    #  \see gui.panel.PanelSummary
    #  \see handleMDIChild()
    def OnCreateSummary(self, event):
        self.handleMDIChild(
            PanelSummary,
            'ViewSummary',
            self.OnCloseSummary,
            _(u"bersicht")
            )

    ## Show and maintain all currencies.
    #
    #  \see gui.panel.PanelCurrency
    #  \see handleMDIChild()
    def OnCreateCurrency(self, event):
        self.handleMDIChild(
            PanelCurrency,
            'ViewCurrency',
            self.OnCloseCurrency,
            _(u"Whrungen")
            )

    ## Show and maintain all exchanges.
    #
    #  \see gui.panel.PanelExchange
    #  \see handleMDIChild()
    def OnCreateExchange(self, event):
        self.handleMDIChild(
            PanelExchange,
            'ViewExchange',
            self.OnCloseExchange,
            _(u"Brsen")
            )

    ## Show and maintain all shares.
    #
    #  \see gui.panel.PanelShare
    #  \see handleMDIChild()
    def OnCreateShare(self, event):
        self.handleMDIChild(
            PanelShare,
            'ViewShare',
            self.OnCloseShare,
            _(u"Wertpapiere")
            )

    ## Show and maintain all transactions.
    #
    #  \see gui.panel.PanelTransaction
    #  \see handleMDIChild()
    def OnCreateTransaction(self, event):
        self.handleMDIChild(
            PanelTransaction,
            'ViewTransaction',
            self.OnCloseTransaction,
            _(u"Transaktionen")
            )

    ## Show and maintain all indices.
    #
    #  \see gui.panel.PanelIndex
    #  \see handleMDIChild()
    def OnCreateIndex(self, event):
        self.handleMDIChild(
            PanelIndex,
            'ViewIndex',
            self.OnCloseIndex,
            _(u"Indices")
            )

    ## Show all open positions.
    #
    #  \see gui.panel.PanelOpenPositionList
    #  \see handleMDIChild()
    def OnCreatePositionList(self, event):
        self.handleMDIChild(
            PanelOpenPositionList,
            'ViewPositionList',
            self.OnClosePositionList,
            _(u"Offene Positionen - Liste")
            )

    ## Show all open positions.
    #
    #  \see gui.panel.PanelOpenPositionDetails
    #  \see handleMDIChild()
    def OnCreatePositionDetails(self, event):
        self.handleMDIChild(
            PanelOpenPositionDetails,
            'ViewPositionDetails',
            self.OnClosePositionDetails,
            _(u"Offene Positionen - Details")
            )

    ## Show all watch list entries
    #
    #  \see gui.panel.PanelWatchList
    #  \see handleMDIChild()
    def OnCreateWatchList(self, event):
        self.handleMDIChild(
            PanelWatchList,
            'ViewWatchList',
            self.OnCloseWatchList,
            _(u"Beobachtungsliste")
            )


    #===================================
    # Functions to cleanup closing views
    #===================================


    ## Cleanup / udate actions on closing summary panel
    #
    #  \see gui.panel.PanelSummary
    def OnCloseSummary(self, event):
        self.closeView('ViewSummary', event)

    ## Cleanup / udate actions on closing currency panel
    #
    #  \see gui.panel.PanelCurrency
    def OnCloseCurrency(self, event):
        self.closeView('ViewCurrency', event)

    ## Cleanup / udate actions on closing exchange panel
    #
    #  \see gui.panel.PanelExchange
    def OnCloseExchange(self, event):
        self.closeView('ViewExchange', event)

    ## Cleanup / udate actions on closing share panel
    #
    #  \see gui.panel.PanelShare
    def OnCloseShare(self, event):
        self.closeView('ViewShare', event)

    ## Cleanup / udate actions on closing transaction panel
    #
    #  \see gui.panel.PanelTransaction
    def OnCloseTransaction(self, event):
        self.closeView('ViewTransaction', event)

    ## Cleanup / udate actions on closing indices panel
    #
    #  \see gui.panel.PanelIndex
    def OnCloseIndex(self, event):
        self.closeView('ViewIndex', event)

    ## Cleanup / udate actions on closing open position panel
    #
    #  \see gui.panel.PanelOpenPositionList
    def OnClosePositionList(self, event):
        self.closeView('ViewPositionList', event)

    ## Cleanup / udate actions on closing open position panel
    #
    #  \see gui.panel.PanelOpenPositionDetails
    def OnClosePositionDetails(self, event):
        self.closeView('ViewPositionDetails', event)

    ## Cleanup / udate actions on closing watch list panel
    #
    #  \see gui.panel.PanelWatch
    def OnCloseWatchList(self, event):
        self.closeView('ViewWatchList', event)

    ## Close a open view and perform all cleanup actions
    def closeView(self, attr, event):
        # uncheck menu entry
        self.viewmenu.Check(self.attr2menu[attr], False)

        # set attribute itself to None
        setattr(self, attr, None)

        # remove child from list open views
        self.views_open.remove(attr)

        # continue event processing
        event.Skip()


    #==========================================================
    # Functions to save and restore the current look of the GUI
    #==========================================================


    ## Save the current configuration before the GUI is closed.
    #
    #  \see saveGUIConfig(), saveSMMConfig(), restoreGUIConfig() and
    #       restoreConfig()
    def OnClose(self, event):
        # save our own config
        self.saveGUIConfig()

        if smm.database.store:
            # check childs to save the configuration
            for name in self.views_open:
                mdi_frame = getattr(self, name, None)
                if not mdi_frame:
                    continue
                child = mdi_frame.GetChildren()[0]
                if hasattr(child, 'saveConfig'):
                    child.saveConfig()

            # save remaining SMM specific GUI settings
            self.saveSMMConfig()

        # continue event processing
        event.Skip()

    ## Restore general GUI configuration 
    #
    #  \see restoreSMMConfig(), saveGUIConfig() and saveSMMConfig()
    def restoreGUIConfig(self):
        self.SetDimensions(
            config.get_item('application/posx'),
            config.get_item('application/posy'),
            config.get_item('application/width'),
            config.get_item('application/height')
            )

    ## Restore GUI configuration of all SMM elements
    #
    #  \see restoreGUIConfig(), saveGUIConfig() and saveSMMConfig()
    def restoreSMMConfig(self):
        # restore open views
        for view in config.get_item("application/open_views"):
            if view == 'ViewSummary':
                self.viewmenu.Check(self.ID_VIEWMENU_SUMMARY, True)
                self.OnCreateSummary(None)
            elif view == 'ViewCurrency':
                self.viewmenu.Check(self.ID_VIEWMENU_CURRENCY, True)
                self.OnCreateCurrency(None)
            elif view == 'ViewExchange':
                self.viewmenu.Check(self.ID_VIEWMENU_EXCHANGE, True)
                self.OnCreateExchange(None)
            elif view == 'ViewShare':
                self.viewmenu.Check(self.ID_VIEWMENU_SHARE, True)
                self.OnCreateShare(None)
            elif view == 'ViewTransaction':
                self.viewmenu.Check(self.ID_VIEWMENU_TRANSACTION, True)
                self.OnCreateTransaction(None)
            elif view == 'ViewIndex':
                self.viewmenu.Check(self.ID_VIEWMENU_INDEX, True)
                self.OnCreateIndex(None)
            elif view == 'ViewPositionList':
                self.viewmenu.Check(self.ID_VIEWMENU_POSITIONLIST, True)
                self.OnCreatePositionList(None)
            elif view == 'ViewPositionDetails':
                self.viewmenu.Check(self.ID_VIEWMENU_POSITIONDETAILS, True)
                self.OnCreatePositionDetails(None)
            elif view == 'ViewWatchList':
                self.viewmenu.Check(self.ID_VIEWMENU_WATCHLIST, True)
                self.OnCreateWatchList(None)

        # reactivate last active view
        name = config.get_item('application/active_view')
        if hasattr(self, name):
            child = getattr(self, name)
            wx.FutureCall(500, child.Activate)
        else:
            logging.debug(
                _(u'''Can't restore last open view, the attribute "%s" doesn't exists.'''),
                name
                )


    ## Store general GUI configuration
    #
    #  \see saveSMMConfig(), restoreGUIConfig() and restoreSMMConfig()
    def saveGUIConfig(self):
        # store size and position of the application
        size = self.GetSize()
        pos = self.GetPosition()
        config.set_item('application/height', size.GetHeight())
        config.set_item('application/width',  size.GetWidth())
        config.set_item('application/posx',   pos.x)
        config.set_item('application/posy',   pos.y) 

    ## Store general GUI configuration
    #
    #  \see saveGUIConfig(), restoreGUIConfig() and restoreSMMConfig()
    def saveSMMConfig(self):
        # store open childs
        config.set_item('application/open_views', self.views_open)

        active = self.GetActiveChild()
        if active:
            name = active.viewName
        else:
            name = ""
        config.set_item('application/active_view', name)

