# -*- coding: Latin-1 -*-
#
# SMM Share and Money Management
# Copyright (c) 2007-2009 Carsten Grohmann
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 2 of the
# License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

## \file gui/config.py
#
# Function to handle configuration settings
#
# \namespace gui::config
# The configuration module handles configuration entries stored
# in a file.
#
# \author Carsten Grohmann <mail (at) carstengrohmann (dot) de>
#
# \par License:
# GPL version 2.0

import ConfigParser
import logging
import os.path
import sys
import types

## \var config
#  Contains the current configuration items
config = {}

## File name of the configuration file
#
#  \see config_file_win and config_file_unix
config_file = ''

## File name of the configuration file on Windows platforms
config_file_win  = "./smmgui.ini"

## File name of the configuration file on Unix platforms
config_file_unix = "./smmguirc"

## Default configuration settings
config_defaults = {}

## \var has_changed
#  Indicate changes at the configuration items
has_changed = False

## Small header added to every written %config file
#
#  Every header line should start with \c # to mark it as a comment line.
header = """\
# Diese Datei wird automatisch erzeugt. Manuelle Aenderungen werden
# ohne Rueckfrage berschrieben.
#
# This is an automatically generated file. Manual changes will be
# overwritten without warning.

"""


## Separator to converts lists into strings and back
list_sep = ' '

config_defaults = {
    # database file of the SQLite database
    "database/file":      {
        "value": "./smm.db",
        "type": "Path"},

    # default encoding of the logfile
    "logging/encoding":      {
        "value": "utf-8",
        "type": "String"},

    # verbosity of the log messages
    "logging/level":         {
        "value": "WARNING",
        "type": "String"},

    # option for the main window
    "application/width":         {
        "value": 600,
        "type": "Int"},
    "application/height":        {
        "value": 400,
        "type": "Int"},
    "application/posx":          {
        "value": 100,
        "type": "Int"},
    "application/posy":          {
        "value": 100,
        "type": "Int"},
    "application/active_view":   {
        "value": "",
        "type": "String"},
    "application/open_views":    {
        "value": [],
        "type": "List"},

    "panel_currency/list-column_widths":     {
        "value": ['100', '100', '-1'],
        "type": "List"},
    "panel_exchange/list-column_widths":     {
        "value": ['90', '-1'],
        "type": "List"},
    "panel_share/list-column_widths":     {
        "value": ['90', '150', '150', '-1', '150', '120', '120', '120', '120', '-1'],
        "type": "List"},
    "panel_transaction/list-column_widths":     {
        "value": ['120', '120', '-1', '150', '90', '120', '120', '120'],
        "type": "List"},
    "panel_index/list-column_widths":     {
        "value": ['90', '150', '150', '-1'],
        "type": "List"},
    "panel_positionlist/lc_openposition-column_widths":       {
        "value": ['20', '-1', '120', '120', '120', '120', '120', '120', '-1'],
            "type": "List"},
    "panel_positionlist/lc_transaction-column_widths": {
        "value": ['120', '120', '120', '120', '120', '120','120', '120'],
        "type": "List"},
    "panel_positionlist/sash_position": {
        "value": 200,
        "type": "Int"},
    "panel_positiondetails/lc_openposition-column_widths":       {
        "value": ['20', '-1'],
        "type": "List"},
    "panel_positiondetails/lc_transaction-column_widths": {
        "value": ['120', '120', '120', '120', '120', '120','120', '120'],
        "type": "List"},
    "panel_positiondetails/sash_position": {
        "value": 100,
        "type": "Int"},
    "panel_watchlist/sash_position": {
        "value": 200,
        "type": "Int"},
    "panel_watchlist/lc_watchposition-column_widths":    {
        "value": ['-1'],
        "type": "List"},
    "panel_watchlist/lc_dailyquotes-column_widths":      {
        "value": ['120', '120', '120', '120', '120', '120', '120'],
        "type": "List"},
    "dlgpositionreport/lc_position-column_widths":       {
        "value": ['120', '120', '120', '120'],
        "type": "List"},
    }

## Initialize the class and load all values
#
#  \param filename  Name of the configuration file or None to use default
#                   settings
def init(filename=None):

    # use some global variables
    global config, config_file, has_changed

    if filename:
        config_file = filename
    elif sys.platform.startswith('win'):
        config_file = config_file_win
    else:
        config_file = config_file_unix

    # normalize file name and remove users ~
    config_file = os.path.abspath(config_file)
    config_file = os.path.normpath(config_file)
    config_file = os.path.expanduser(config_file)

    # set change flag to false
    has_changed = False

    # clear config
    config = {}

    # open config file parser
    if os.path.isfile(config_file):
        # load values from file
        load()
    else:
        logging.warning(
            _(u'Konnte die Konfigurationdatei %s nicht finden. ' \
              u'Initialisiere Standardwerte stattdessen.'),
            config_file
            )
        # initialize configuration from default values
        for key in config_defaults.keys():
            value = config_defaults[key]['value']
            set_item(key, value)

## Set a configuration value
#
#  \param key   Configuration key <c>section/option</c>
#  \param value New value
#  \exception TypeError  If item's type doesn't fit the excepted type
#  \exception ValueError For unknown configuration items
def set_item(key, value):

    # use some global variables
    global has_changed

    key = key.lower()

    # check if key is a valid configuration item
    if not config_defaults.has_key(key):
        raise ValueError, "Unknown configuration item: %s!" % key

    # do nothing if the value are already set
    if config.has_key(key) and value == config[key]:
        return

    # check types
    if config_defaults[key]["type"] == "Int":
        try:
            if type(value) is not types.IntType:
                value = int(value)
        except:
            raise TypeError, \
                "Wrong type for configuration item %s (got: %s, expect: %s)" % (
                    key,
                    type(value),
                    types.IntType,
                    )

    elif config_defaults[key]["type"] == "Boolean":
        if type(value) is not types.BooleanType:
            raise TypeError, \
                "Wrong type for configuration item %s (got: %s, expect: %s)" % (
                    key,
                    type(value),
                    types.BooleanType,
                    )

    elif config_defaults[key]["type"] == "List":
        if type(value) is not types.ListType:
            raise TypeError, \
                "Wrong type for configuration item %s (got: %s, expect: %s)" % (
                    key,
                    type(value),
                    types.ListType,
                    )

    elif config_defaults[key]["type"] in ["String", "Path"]:
        if type(value) is types.UnicodeType:
            try:
                temp = value.encode('ascii')
                value = temp
            except UnicodeEncodeError:
                raise TypeError, \
                  "Can not convert Unicode value '%s' for item %s into ascii" % (
                      value.encode('ascii', 'replace'),
                      key,
                      )
        elif type(value) is not types.StringType:
            raise TypeError, \
                "Wrong type for configuration item %s (got: %s, expect: %s)" % (
                    key,
                    type(value),
                    types.StringType,
                    )

        # normalize file name and remove users ~
        if config_defaults[key]["type"] == "Path":
            value = os.path.abspath(value)
            value = os.path.normpath(value)
            value = os.path.expanduser(value)

    else:
        raise TypeError, "Unknown configuration type %s for item %s" % (
                config_defaults[key]["type"],
                key,
                )


    # set new item
    config[key] = value
    has_changed = True

## Returns a configuration value.
#
#  \return The value of the key, if it exists otherwiese None
#  \exception ValueError If the requested key is unkown
def get_item(key):
    key = key.lower()

    # get value from current config or from defaults
    if config.has_key(key):
        value = config[key]
    elif config_defaults.has_key(key):
        value = config_defaults[key]["value"]
    else:
        raise ValueError, "Unknown configuration item: %s!" % key

    # normalize file name and remove users ~
    if config_defaults[key]["type"] == "Path":
        value = os.path.abspath(value)
        value = os.path.normpath(value)
        value = os.path.expanduser(value)

    return value

## Load the configuration from a file
#
#  \exception TypeError If the type of the configuration item is unknown
def load():

    # use some global variables
    global config, has_changed

    try:
        parser = ConfigParser.ConfigParser()
        parser.read(config_file)

        # clear existing configuration
        config = {}

        # read all configuration entries with default values
        for key in config_defaults:
            # split in two parts
            (section, option) = key.split("/", 1)

            # skip keys without a value in the config file
            if not parser.has_option(section, option):
                continue

            # read value from parser object
            if config_defaults[key]["type"] == "Int":
                value = parser.getint(section, option)

            elif config_defaults[key]["type"] == "Boolean":
                value = parser.getboolean(section, option)

            elif config_defaults[key]["type"] == "List":
                value = parser.get(section, option)
                value = value.split(list_sep)

            elif config_defaults[key]["type"] == "String":
                value = parser.get(section, option)

            elif config_defaults[key]["type"] == "Path":
                value = parser.get(section, option)
                value = os.path.abspath(value)
                value = os.path.normpath(value)
                value = os.path.expanduser(value)

            else:
                raise TypeError, "Unknown configuration type!"

            # store value
            config[key] = value

    except:
        logging.exception(
            'Unexpected error during loading configuration items'
            )

    # set the change flag back
    has_changed = False

## Store configuration into a file
#
#  The unchanged values will not write to the configuration file.
#  \param force   Forces writing to disk also if the configuration is
#                 unchanged
#  \param saveAll Save all configuration items, false means to save
#                 the changed items only
def save(force=False, saveAll=False):

    # use some global variables
    global has_changed

    # nothing to do?
    if not has_changed and not force:
        return

    # store configuration in a ConfigParser object
    parser = ConfigParser.ConfigParser()
    for key in config_defaults.keys():

        # get value from current config or from defaults
        if config.has_key(key):
            value = config[key]
        elif config_defaults.has_key(key) and saveAll:
            value = config_defaults[key]["value"]
        else:
            continue

        # process special kinds of items like lists
        if config_defaults[key]["type"] == "List":
            value = list_sep.join(value)

        (section, option) = key.split("/")

        # add section
        if not parser.has_section(section):
            parser.add_section(section)

        # set configuration into ConfigParser object
        parser.set(section, option, value)

    # save all to file
    try:
        configfile = file(config_file, "w")
        if header:
            configfile.write(header)
        parser.write(configfile)
        configfile.close()
    except:
        logging.exception('Error during writing configuration items')

    # set change flag back to false
    has_changed = False
