# -*- coding: Latin-1 -*-
#
# SMM Share and Money Management
# Copyright (c) 2007-2009 Carsten Grohmann
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 2 of the
# License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

## \file gui/reports.py
#
# \brief Create %reports about the portfolio
#
# \namespace gui::reports
# \brief Create detailed %reports about of positions or according to the
# calendar.
#
# The function used the Mako (http://www.makotemplates.org) to render
# those %reports. Mako is a lightweight template library and it's written
# in pure Python.
#
# \author Carsten Grohmann <mail (at) carstengrohmann (dot) de>
#
# \par License:
# GPL version 2.0

# import other general python modules
import datetime
import decimal

# import 3rd party libs
from storm.locals import Asc
from mako.template import Template
from mako.runtime import Context

# import main lib
import smm
import helper

## HTML template for a position report
#
#  \see createPositionReport()
position_report_tmpl = """\
<html>
<h1>${share_name}</h1>

<table>
  <tr><td>Name</td><td>${share_name}</td></tr>
  <tr><td>Gattung</td><td>${kind}</td></tr>
  %if share_isin:
    <tr><td>ISIN</td><td>${share_isin}</td></tr>
  %endif
  %if share_wkn:
    <tr><td>WKN</td><td>${share_wkn}</td></tr>
  %endif
  %if share_symbol:
    <tr><td>Symbol</td><td>${share_symbol}</td></tr>
  %endif
</table>

<br>
<br>
<h2>Positionsdetails</h2>

<table>
  <tr>
    <td>Status</td>
    <td align="right">${pos_state}</td>
    <td>&nbsp;</td>
  </tr>
  %if pos_open == True:
    <tr>
      <td>Bestand</td>
      <td align="right">${pos_count}&nbsp;St&uuml;ck</td>
      <td>&nbsp;</td>
    </tr>
    <tr>
      <td>Volumen</td>
      <td align="right">${pos_volume}</td>
      <td>&nbsp;</td>
    </tr>
    <tr>
      <td>Letzter Kurs</td>
      <td align="right">${pos_quote} vom ${pos_quote_date}</td>
      <td>&nbsp;</td>
    </tr>
  %endif
  <tr>
    <td>&nbsp;</td>
    <td align="right">Ohne Geb&uuml;hren</td>
    <td align="right">Mit Geb&uuml;hren</td>
  </tr>
  <tr>
    <td>GuV absolut</td>
    <td align="right">${profit_abs}</td>
    <td align="right">${profit_abs_fee}</td>
  </tr>
  <tr>
    <td>GuV relativ</td>
    <td align="right">${profit_rel}</td>
    <td align="right">${profit_rel_fee}</td>
  </tr>
  <tr>
    <td valign="top">Bemerkung</td>
    <td colspan="2">
    %if pos_notes:
        <% first = True %>
        %for note in pos_notes:
          %if first:
            <% first = False %>
          %else:
            <br>
          %endif
          ${note}
        %endfor
    %else:
      &nbsp;
    %endif
    </td>
  </tr>
</table>

<br>
<br>
<h2>Transaktionen</h2>

<table border="1">
  <tr align="center">
    <td>Datum</td>
    <td>Art</td>
    <td>Anzahl</td>
    <td>Kurs</td>
    <td>Betrag</td>
    <td>Geb&uuml;hr</td>
    <td>B&ouml;rsenplatz</td>
  </tr>
  %for trans in transactions:
  <tr>
    <td>${trans['date']}</td>
    <td>${trans['kind']}</td>
    <td align="right">${trans['count']}</td>
    <td align="right">${trans['quote']}</td>
    <td align="right">${trans['volume']}</td>
    <td align="right">${trans['fee']}</td>
    <td>${trans['exchange']}</td>
  </tr>
 %endfor
  <tr>
    <td>Summe</td>
    <td></td>
    <td></td>
    <td></td>
    <td></td>
    <td align="right">${pos_fee}</td>
    <td></td>
  </tr>
</table>
<hr>
  <table width="100%" border="0" style="font-size:50%">
    <tr>
      <td align="left">Share and Money Management</td>
      <td align="right">${creation_timestamp}</td>
    </tr>
  </table>
</html>
"""

## HTML template for a report of the whole portfolio
#
#  \see createPortfolioReport()
portfolio_report_tmpl = """\
<html>
<h1>Portfolio&uuml;bersicht von ${start_date} bis ${end_date}</h1>

<br>
<br>
<h2>Transaktions&uuml;bersicht</h2>
<table border="1">
  %for pos in positions:
  <tr>
    <td colspan="7">
      <br><br><br>
      <strong>
      ${pos['name']}
      %if pos['isin']:
        | ISIN: ${pos['isin']}
      %endif
      %if pos['wkn']:
        | WKN: ${pos['wkn']}
      %endif
      %if pos['symbol']:
        | Symbol: ${pos['symbol']}
      %endif
      </strong>
    </td>
  </tr>
  <tr align="center">
    <td>Datum</td>
    <td>Art</td>
    <td>Anzahl</td>
    <td>Kurs</td>
    <td>Betrag</td>
    <td>Geb&uuml;hr</td>
    <td>B&ouml;rsenplatz</td>
  </tr>
  %for trans in pos['transactions']:
  <tr>
    <td>${trans['date']}</td>
    <td>${trans['kind']}</td>
    <td align='right'>${trans['count']}</td>
    <td align='right'>${trans['quote']}</td>
    <td align='right'>${trans['volume']}</td>
    <td align='right'>${trans['fee']}</td>
    <td>${trans['exchange']}</td>
  </tr>
  %endfor
  <tr>
    <td>Summe</td>
    <td></td>
    <td></td>
    <td></td>
    <td></td>
    <td align='right'>${pos['fee']}</td>
    <td></td>
  </tr>
  %endfor
</table>

%if fees:
<!-- Orderkosten getrennt nach Waehrungen -->
<br>
<br>
<h2>Geb&uuml;hren</h2>
<table>
  %for pos in fees:
  <tr>
    <td>Geb&uuml;hren ${pos['code']}</td>
    <td align='right'>${pos['value']}</td>
  </tr>
  %endfor
  </table>
%endif

<br>
<br>
<h2>Realisierungs&uuml;bersicht</h2>
<table border="1" valign="top">
  %for pos in realised:
  <tr>
    <td colspan="10">
      <br><br><br>
      <strong>
      ${pos['name']}
      %if pos['isin']:
        | ISIN: ${pos['isin']}
      %endif
      %if pos['wkn']:
        | WKN: ${pos['wkn']}
      %endif
      %if pos['symbol']:
        | Symbol: ${pos['symbol']}
      %endif
      </strong>
    </td>
  </tr>
  <tr align="center">
    <td>Lfd. Nr.</td>
    <td>Datum</td>
    <td>Art</td>
    <td>Anzahl</td>
    <td>Kurs</td>
    <td>Betrag</td>
    <td>GuV</td>
    <td>Geb&uuml;hr</td>
    <td>GuV gesamt</td>
    <td>Verkauf von Nr. (Haltedauer)</td>
  </tr>
  %for trans in pos['transactions']:
  <tr>
    <td align="center">${trans['line']}</td>
    <td>${trans['date']}</td>
    <td>${trans['kind']}</td>
    <td align='right'>${trans['count']}</td>
    <td align='right'>${trans['quote']}</td>
    <td align='right'>${trans['volume']}</td>
    <td align='right'>${trans['profit']}</td>
    <td align='right'>${trans['fee']}</td>
    <td align='right'>${trans['profit_total']}</td>
    <td>
      %if trans.has_key('comments'):
        <% first = True %>
        %for comment in trans['comments']:
          %if first:
            <% first = False %>
          %else:
            <br>
          %endif
          ${comment}
        %endfor
      %endif
    </td>
  </tr>
  %endfor
  <tr>
    <td>Summe</td>
    <td></td>
    <td></td>
    <td align='right'>${pos['pos_count']}</td>
    <td></td>
    <td></td>
    <td align='right'>${pos['pos_profit']}</td>
    <td align='right'>${pos['pos_fee']}</td>
    <td align='right'>${pos['pos_profit_total']}</td>
    <td></td>
  </tr>
  %endfor
</table>

%if profit:
<!-- Realisierungen getrennt nach Waehrungen -->
<br>
<br>
<h2>Realisierungen</h2>
<table>
  %for pos in profit:
  <tr>
    <td>Geb&uuml;hren ${pos['code']}</td>
    <td align='right'>${pos['fee']}</td>
  </tr>
  <tr>
    <td>GuV</td>
    <td align='right'>${pos['profit']}</td>
  </tr>
  <tr>
    <td>GuV gesamt</td>
    <td align='right'>${pos['profit_total']}</td>
  </tr>
  %endfor
  </table>
%endif

<hr>
  <table width="100%" border="0" style="font-size:50%">
    <tr>
      <td align="left">Share and Money Management</td>
      <td align="right">${creation_timestamp}</td>
    </tr>
  </table>
</html>
"""

## Create a report of a single position
#
#  The report is created using the HTML template in \ref position_report_tmpl
#  and stored in \c dest.
#
#  \param pos  The \ref smm.objects.Position "position" to report
#  \param dest A file-like object
#
#  \see position_report_tmpl
def createPositionReport(pos, dest):
    kw = {}

    # general information
    kw['share_name']   = helper.formatShare(pos.share)
    kw['share_isin']   = pos.share.isin
    kw['share_wkn']    = pos.share.wkn
    kw['share_symbol'] = pos.share.symbol
    kw['creation_timestamp'] = helper.formatDate(
        datetime.datetime.now(),
        '%d.%m.%Y %H:%M'
        )
    kw['kind']   = helper.formatShareKind(pos.share)

    kw['pos_open'] = pos.open
    if pos.open:
        kw['pos_state']  = _(u'aktiv')
        kw['pos_count']  = helper.formatDecimal(pos.getShareCount(), 3)
        kw['pos_volume'] = helper.formatMoney(pos.getVolume(), pos.currency, 2)
        quote, date = pos.getLastQuote()
        kw['pos_quote'] = helper.formatMoney(quote, pos.currency)
        kw['pos_quote_date'] = helper.formatDate(date)
    else:
        kw['pos_state']  = _(u'geschlossen')

    if pos.notes:
        kw['pos_notes'] = pos.notes.split('\n')

    # calc profit without fee
    profit_abs, profit_rel = pos.getPerf()
    kw['profit_abs'] = helper.formatMoney(profit_abs, pos.currency, 2)
    kw['profit_rel'] = helper.formatPercent(profit_rel, 2)

    # calc profit with fee
    profit_abs_fee, profit_rel_fee = pos.getPerf(withFee=True)
    kw['profit_abs_fee'] = helper.formatMoney(profit_abs_fee, pos.currency, 2)
    kw['profit_rel_fee'] = helper.formatPercent(profit_rel_fee, 2)

    # details of all transactions
    kw['transactions'] = []
    transactions = pos.transactions.order_by(Asc(smm.Transaction.date))
    pos_fee = decimal.Decimal('0')
    for trans in transactions:
        t = {}
        t['count']    = helper.formatDecimal(trans.count, 3)
        t['date']     = helper.formatDate(trans.date)
        t['exchange'] = helper.formatExchange(trans.exchange)
        if trans.fee:
            t['fee']   = helper.formatMoney(trans.fee, pos.currency, 2)
            pos_fee += trans.fee
        else:
            t['fee']   = helper.formatMoney(
                decimal.Decimal('0'),
                pos.currency,
                2)
        t['kind']   = helper.formatTransactionKind(trans)
        t['quote']  = helper.formatMoney(trans.value, pos.currency)
        t['volume'] = helper.formatMoney(
            trans.getVolume(withFee=False),
            pos.currency,
            2
            )
        kw['transactions'].append(t)

    # additional stuff
    kw['pos_fee'] = helper.formatMoney(pos_fee, pos.currency, 2)

    # create and render template
    template = Template(
        position_report_tmpl,
        default_filters=['entity', 'trim'],
        )
    ctx = Context(dest, **kw)
    template.render_context(ctx)


## Create a report of portfolio changes between start and end date
#
#  The report is created using the HTML template in \ref portfolio_report_tmpl
#  and stored in \c dest.
#
#  \param start Start date
#  \param end   End date
#  \param dest  A file-like object
#
#  \see portfolio_report_tmpl
def createPortfolioReport(start, end, dest):
    kw = {}

    # general information
    kw['start_date'] = helper.formatDate(start)
    kw['end_date']   = helper.formatDate(end)
    kw['creation_timestamp'] = helper.formatDate(
        datetime.datetime.now(),
        '%d.%m.%Y %H:%M'
        )

    kw['fees']       = []
    kw['positions']  = []
    kw['realised']   = []
    kw['profit']     = []
    all_fees         = {}
    all_realisations = {}

    # get all affected positions
    positions = sorted(
        smm.Position.getInPeriod(start, end),
        key=lambda pos: pos.share.name
        )

    # details of all transactions
    for pos in positions:
        p = {}

        # indicates realisations (means position has sell or divident
        # transactions)
        has_realised = False

        p['name']   = helper.formatShare(pos.share)
        p['isin']   = pos.share.isin
        p['wkn']    = pos.share.wkn
        p['symbol'] = pos.share.symbol

        fee = decimal.Decimal('0')

        # get all transactions for this position
        pos_transactions = smm.Transaction.getInPeriod(start, end, pos)

        p['transactions'] = []

        # process all transactions of the current position
        for trans in pos_transactions:
            t = {}
            t['count']    = helper.formatDecimal(trans.count, 3)
            t['date']     = helper.formatDate(trans.date)
            t['exchange'] = helper.formatExchange(trans.exchange)
            if trans.fee:
                t['fee']   = helper.formatMoney(trans.fee, pos.currency, 2)
                fee += trans.fee
            else:
                t['fee']   = helper.formatMoney(
                    decimal.Decimal('0'),
                    pos.currency,
                    2)
            t['kind']   = helper.formatTransactionKind(trans)
            t['quote']  = helper.formatMoney(trans.value, pos.currency)
            t['volume'] = helper.formatMoney(
                trans.getVolume(withFee=False),
                pos.currency,
                2
                )
            p['transactions'].append(t)

            # check for realisation
            if trans.kind not in ['buy', 'split']:
                has_realised = True

        # fill general data of realised position with existing data
        if has_realised:
            details_realised = {}
            for attr in ['name', 'isin', 'wkn', 'symbol']:
                details_realised[attr] = p[attr]
            details_realised['pos'] = pos
            kw['realised'].append(details_realised)

        # format position fee summary
        p['fee'] = helper.formatMoney(fee, pos.currency, 2)

        kw['positions'].append(p)

        # add position fee to global fee list
        if pos.currency not in all_fees:
            all_fees[pos.currency] = decimal.Decimal('0')
        all_fees[pos.currency] += fee

    # sort fees and create data structure
    all_fees_sorted = sorted(
        all_fees,
        key=lambda obj: obj.code,
        )

    for currency in all_fees_sorted:
        f = {}
        f['code']  = currency.code
        f['value'] = helper.formatMoney(
            all_fees[currency],
            currency,
            2
            )
        kw['fees'].append(f)

    # process realised positions
    for details in kw['realised']:

        pos = details['pos']
        del details['pos']

        # fee overall transactions
        pos_fee = decimal.Decimal('0')

        # internal list of transaction specific information to calculate profit
        trans_buys = []

        line             = 0
        pos_count        = decimal.Decimal('0')
        pos_profit       = decimal.Decimal('0')
        pos_profit_total = decimal.Decimal('0')

        # process all transactions for this position
        details['transactions'] = []
        transactions = pos.transactions.order_by(Asc(smm.Transaction.date))

        for trans in transactions:

            # skip transcations newer than end date
            if trans.date > end:
                continue

            # initialise loop internal variables
            line += 1
            t     = {}

            t['line']     = line
            t['comment']  = u''
            t['count']    = helper.formatDecimal(trans.count, 3)
            t['date']     = helper.formatDate(trans.date)
            tprofit       = decimal.Decimal('0')
            tprofit_total = decimal.Decimal('0')

            if trans.fee:
                t['fee']   = helper.formatMoney(trans.fee, pos.currency, 2)
                pos_fee += trans.fee
            else:
                t['fee']   = helper.formatMoney(
                    decimal.Decimal('0'),
                    pos.currency,
                    2)
            t['kind']   = helper.formatTransactionKind(trans)
            t['quote']  = helper.formatMoney(trans.value, pos.currency)
            t['volume'] = helper.formatMoney(
                trans.getVolume(withFee=False),
                pos.currency,
                2
                )

            # handle transactions
            if trans.kind == 'dividend':
                tprofit = trans.count * trans.value
                tprofit_total = tprofit
                if trans.fee:
                    tprofit_total -= trans.fee

            elif trans.kind == 'buy':
                # store all needed data in a seprate record to prevent
                # manupulation of the original transaction data
                tdata = {
                    'line':        line,
                    'count':       trans.count,
                    'date':        trans.date,
                    'fee':         trans.fee,
                    'remaining':   trans.count,
                    'value':       trans.value,
                    }
                trans_buys.append(tdata)

            elif trans.kind == 'sell':

                # count of sell transactions are negative values but we
                # convert to positive values to simplify code
                count = abs(trans.count)

                lines = []

                while count > decimal.Decimal('0') and trans_buys:

                    # some variables used to calculate the profit
                    # count  - number of shares remaining in the current
                    #          sell transaction
                    # scount - number of shares to process within this
                    #          loop

                    # get first entry from trainsaction details
                    tdata = trans_buys[0]
                    lines.append(u'%d (%s)' % (
                        tdata['line'],
                        helper.formatDuration(
                            abs(trans.getHoldingPeriod(tdata['date']))
                            )
                        ))

                    # reduce share count by number of processed shares
                    if tdata['remaining'] >= count:
                        tdata['remaining'] -= count
                        scount = count
                    else: # tdata['remaining'] < count
                        scount = tdata['remaining']
                        tdata['remaining'] = decimal.Decimal('0')

                    # calculate performance components
                    costs = scount * tdata['value']
                    earnings = scount * trans.value
                    tprofit += earnings - costs
                    tprofit_total += earnings - costs

                    # calucate proportional fee
                    buy_fee_ratio  = scount / tdata['count']
                    sell_fee_ratio = abs(scount / trans.count)
                    if tdata['fee']:
                        tprofit_total -= tdata['fee'] * buy_fee_ratio
                    if trans.fee:
                        tprofit_total -= trans.fee * sell_fee_ratio

                    # remove fully processed buy transactions from the list
                    if tdata['remaining'] == decimal.Decimal('0'):
                        del trans_buys[0]

                    # decrease shares to process by processes shares
                    count -= scount

                # prepare comment to refer to the buy transactions
                t['comments'] = lines

            elif trans.kind == 'split':
                # Every split has two transactions, the first decreases
                # the share count and the second increases the count
                # It's like a sell and a following buy transaction
                if trans.count < decimal.Decimal('0'):
                    trans_buys.append(trans)
                elif trans.count > decimal.Decimal('0'):
                    sell = trans_buys[-1]
                    buy = trans
                    del trans_buys[-1]

                    # adapt existing transactions
                    factor = buy.count / abs(sell.count)
                    for tdata in trans_buys:
                        tdata['count'] *= factor
                        tdata['value'] /= factor
                        tdata['remaining'] *= factor

                # reduce total profit by transaction fee
                if trans.fee:
                    tprofit_total -= trans.fee

            # format transaction profit
            if tprofit:
                pos_profit += tprofit

                t['profit'] = helper.formatMoney(
                    tprofit,
                    pos.currency,
                    2
                    )
            else:
                t['profit'] = u''

            # format transaction profit incl. fees
            if tprofit_total:
                pos_profit_total += tprofit_total

                t['profit_total'] = helper.formatMoney(
                tprofit_total,
                    pos.currency,
                    2
                    )
            else:
                t['profit_total'] = u''

            # fill remaining fields
            details['transactions'].append(t)
            pos_count += trans.count

        # format overall fee
        details['pos_fee'] = helper.formatMoney(pos_fee, pos.currency, 2)

        # format position profit
        if pos_profit:
            details['pos_profit'] = helper.formatMoney(pos_profit, pos.currency, 2)
        else:
            details['pos_profit'] = u''

        # format position profit incl. fees
        if pos_profit_total:
            details['pos_profit_total'] = helper.formatMoney(
                pos_profit_total,
                pos.currency,
                2
                )
        else:
            details['pos_profit_total'] = u''

        # format remaining number of shares
        details['pos_count'] = helper.formatDecimal(pos_count, 3)

        # add profit and fees of all realised position to a global list
        if pos.currency not in all_realisations:
            all_realisations[pos.currency] = {
                'fee':          decimal.Decimal('0'),
                'profit':       decimal.Decimal('0'),
                'profit_total': decimal.Decimal('0'),
                }
        all_realisations[pos.currency]['fee']          -= pos_fee
        all_realisations[pos.currency]['profit']       += pos_profit
        all_realisations[pos.currency]['profit_total'] += pos_profit_total


    # sort fees and create data structure
    all_realisations_sorted = sorted(
        all_realisations,
        key=lambda obj: obj.code,
        )

    for currency in all_realisations_sorted:
        f = {}
        f['code']  = currency.code
        for attr in ['fee', 'profit', 'profit_total']:
            f[attr] = helper.formatMoney(
                all_realisations[currency][attr],
                currency,
                2
                )
        kw['profit'].append(f)

    # create and render template
    template = Template(
        portfolio_report_tmpl,
        default_filters=['entity', 'trim'],
        )
    ctx = Context(dest, **kw)
    template.render_context(ctx)
