# -*- coding: Latin-1 -*-
#
# SMM Share and Money Management
# Copyright (c) 2007-2009 Carsten Grohmann
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 2 of the
# License.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

## \file setup.py
#
# \brief Setup script to distribute SMM
# 
# \author Carsten Grohmann <mail (at) carstengrohmann (dot) de>
# 
# \par License:
# GPL version 2.0

from distutils.core import setup
import glob 
from types import *
import os

# distutils sdisk command is broken because it doesn't copy data_files
# Bug: http://bugs.python.org/issue2279
def add_defaults_fixed(self):
    """Add all the default files to self.filelist:
      - README or README.txt
      - setup.py
      - test/test*.py
      - all pure Python modules mentioned in setup script
      - all files pointed by package_data (build_py)
      - all files defined in data_files.
      - all files defined as scripts.
      - all C sources listed as part of extensions or C libraries
        in the setup script (doesn't catch C headers!)
    Warns if (README or README.txt) or setup.py are missing; everything
    else is optional.
    """

    standards = [('README', 'README.txt'), self.distribution.script_name]
    for fn in standards:
        if type(fn) is TupleType:
            alts = fn
            got_it = 0
            for fn in alts:
                if os.path.exists(fn):
                    got_it = 1
                    self.filelist.append(fn)
                    break

            if not got_it:
                self.warn("standard file not found: should have one of " +
                          string.join(alts, ', '))
        else:
            if os.path.exists(fn):
                self.filelist.append(fn)
            else:
                self.warn("standard file '%s' not found" % fn)

    optional = ['test/test*.py', 'setup.cfg']
    for pattern in optional:
        files = filter(os.path.isfile, glob.glob(pattern))
        if files:
            self.filelist.extend(files)

    # build_py is used to get:
    #  - python modules
    #  - files defined in package_data
    build_py = self.get_finalized_command('build_py')

    # getting python files
    if self.distribution.has_pure_modules():
        self.filelist.extend(build_py.get_source_files())

    # getting package_data files
    # (computed in build_py.data_files by build_py.finalize_options)
    for pkg, src_dir, build_dir, filenames in build_py.data_files:
        for filename in filenames:
            self.filelist.append(os.path.join(src_dir, filename))

    # getting distribution.data_files
    if self.distribution.has_data_files():
        for dirname, filenames in self.distribution.data_files:
            for filename in filenames:
                self.filelist.append(os.path.join(dirname, filename))

    if self.distribution.has_ext_modules():
        build_ext = self.get_finalized_command('build_ext')
        self.filelist.extend(build_ext.get_source_files())

    if self.distribution.has_c_libraries():
        build_clib = self.get_finalized_command('build_clib')
        self.filelist.extend(build_clib.get_source_files())

    if self.distribution.has_scripts():
        build_scripts = self.get_finalized_command('build_scripts')
        self.filelist.extend(build_scripts.get_source_files())

# Replace old implementation by the new own
import distutils.command.sdist
distutils.command.sdist.sdist.add_defaults = add_defaults_fixed

def is_package(path):
    return (
        os.path.isdir(path) and
        os.path.isfile(os.path.join(path, '__init__.py'))
        )

def find_packages(path, base="" ):
    """ Find all packages in path """
    packages = {}
    for item in os.listdir(path):
        dir = os.path.join(path, item)
        if is_package( dir ):
            if base:
                module_name = "%(base)s.%(item)s" % vars()
            else:
                module_name = item
            packages[module_name] = dir
            packages.update(find_packages(dir, module_name))
    return packages

scripts = ['smmgui.py', 'smmgui.pyw', 'smmshell.py', 'test.py',]
data_files = [[
        '',
        ['ChangeLog', 'COPYING.GPL2', 'COPYING.GPL2.de', 'Doxyfile',
         'Makefile', 'smm.nsi', ],
    ],[
       '',
       glob.glob('doc/*'),
    ],[
       '',
       glob.glob('ui/*'),
    ]]
packages = find_packages(".").keys()

py_modules = ['BeautifulSoup', ]

classifiers = """\
Development Status :: 4 - Beta
License :: OSI Approved :: GNU General Public License (GPL)
Natural Language :: German
Operating System :: MacOS :: MacOS X
Operating System :: Microsoft :: Windows :: Windows 95/98/2000
Operating System :: Microsoft :: Windows :: Windows NT/2000
Operating System :: POSIX
Operating System :: Unix
Programming Language :: Python :: 2.5
Topic :: Office/Business :: Financial
"""

import smm
version = smm.version

setup(
    name='smm',
    version=version,
    author='Carsten Grohmann',
    author_email='mail@carstengrohmann.de',
    classifiers=classifiers.split("\n"),
    url='http://www.carstengrohmann.de/smm.html',
    description='Programm zum Verwalten eines privaten Portfolio',
    long_description="""\
Mit SMM knnen Sie die einzelnen Positionen Ihres Depots in einem Programm
verwalten. Dazu werden alle Wertpapiere und die dazugehrigen Transaktionen
verwaltet. Damit haben Sie jederzeit einen berblick ber Ihre Anlage.

Weiterhin knnen Sie die Entwicklung einzelner Positionen, seien es
aktuelle offene oder auch geschlossenen Positionen ansehen oder sich einen
Bericht zum Ausdrucken erstellen lassen. Sie knnen sich auch die
Entwicklung Ihres einzelnen Bestandes ber einen frei whlbaren Zeitraum
darstellen lassen.

Es lt sich eine Liste (s.g. Watchlist) mit interessanten Wertpapieren,
die Sie beobachten mchten, parallel zum aktuellen Bestand fhren.

Das Programm bietet eine komfortable Oberflche, ber die sich fast alle 
Aktionen ttigen lassen. Es sind noch nicht alle Mglichkeiten
implementiert, so ist es zum Beispiel derzeit nicht mglich Transaktionen
nachtrglich ber die Oberflche zu ndern.

Eine Unterscheidung zwischen verschiedenen Depots ist nicht mglich.
Ebenso wenig existieren Schnittstellen zu Banken, weder um das Portfolio
automatisch abzugleichen noch um Transaktionen zu ttigen.
""",
    license='GNU General Public License Version 2',
    platforms = ['WIN32', 'OSX', 'POSIX'],
    scripts=scripts,
    packages=packages,
    py_modules=py_modules,
    data_files=data_files,
    )
