# -*- coding: Latin-1 -*-
#
# SMM Share and Money Management
# Copyright (c) 2007-2010 Carsten Grohmann
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 2 of the
# License.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

## \file smm/db_setup.py
#
# Functions and data to setup a fresh database
#
# \namespace smm::db_setup
# Database schema and the initial data set as well as functions to setup
# a fresh database using the schema and initial data set
#
# \author Carsten Grohmann <mail (at) carstengrohmann (dot) de>
#
# \par License:
# GPL version 2.0

# import project modules
from storm.locals import *

# import project modules
import database
import objects

## \brief Symbols and names of the most known exchanges
#  \note  The list of exchanges are from http://finance.yahoo.com/exchanges
exchange_list = {
    u'AS':  u'Amsterdam Stock Exchange, Netherlands',
    u'AX':  u'Australian Stock Exchange',
    u'BA':  u'Buenos Aires Stock Exchange, Argentina',
    u'BC':  u'Barcelona Stock Exchange, Spain',
    u'BE':  u'Berlin, Germany',
    u'BI':  u'Bilbao Stock Exchange, Spain',
    u'BM':  u'Bremen, Germany',
    u'BO':  u'Bombay Stock Exchange, India',
    u'CBT': u'Chicago Board of Trade, United States of America',
    u'CME': u'Chicago Mercantile Exchange, United States of America',
    u'CMX': u'New York Commodities Exchange, United States of America',
    u'CO':  u'Copenhagen Stock Exchange, Denmark',
    u'D':   u'Dsseldorf, Germany',
    u'DE':  u'XETRA, Germany',
    u'F':   u'Frankfurt, Germany',
    u'HA':  u'Hannover, Germany',
    u'H':   u'Hamburg, Germany',
    u'HK':  u'Hong Kong Stock Exchange, Hong Kong',
    u'JK':  u'Jakarta Stock Exchange, Indonesia',
    u'KQ':  u'KOSDAQ, South Korea',
    u'KS':  u'Korea Stock Exchange, South Korea',
    u'L':   u'London Stock Exchange, United Kingdom',
    u'MA':  u'Madrid Stock Exchange, Spain',
    u'MC':  u'Madrid SE C.A.T.S., Spain',
    u'MF':  u'Madrid Fixed Income Market, Spain',
    u'MI':  u'Milan Stock Exchange, Italy',
    u'MU':  u'Mnchen, Germany',
    u'MX':  u'Mexico Stock Exchange, Mexico',
    u'NS':  u'National Stock Exchange of India, India',
    u'NYB': u'New York Board of Trade, United States of America',
    u'NYM': u'New York Mercantile Exchange, United States of America',
    u'NZ':  u'New Zealand Stock Exchange, New Zealand',
    u'OB':  u'OTC Bulletin Board Market, United States of America',
    u'OL':  u'Oslo Stock Exchange, Norway',
    u'PA':  u'Paris Stock Exchange, France',
    u'PK':  u'Pink Sheets, United States of America',
    u'SA':  u'Sao Paolo Stock Exchange, Brazil',
    u'SG':  u'Stuttgart, Germany',
    u'SI':  u'Singapore Stock Exchange, Singapore',
    u'SS':  u'Shanghai Stock Exchange, China',
    u'ST':  u'Stockholm Stock Exchange, Sweden',
    u'SW':  u'Swiss Exchange, Switzerland',
    u'SZ':  u'Shenzhen Stock Exchange, China',
    u'TA':  u'Tel Aviv Stock Exchange, Israel',
    u'TO':  u'Toronto Stock Exchange, Canada',
    u'TWO': u'Taiwan OTC Exchange, Taiwan',
    u'TW':  u'Taiwan Stock Exchange, Taiwan',
    u'VA':  u'Vienna Stock Exchange, Austria',
    u'V':   u'TSX Venture Exchange, Canada',
    }


## \brief List the most known currencies
currencies_list = [
    (u'CHF', u'Fr.', u'Schweizer Franken'),
    (u'RUB', None,   u'Russischer Rubel'),
    (u'EUR', u'\N{EURO SIGN}',   u'Euro'),
    (u'GBP', u'\N{POUND SIGN}',  u'Pfund Sterling'),
    (u'USD', u'\N{DOLLAR SIGN}', u'US-Dollar'),
    (u'JPY', u'\N{YEN SIGN}',    u'Yen'),
    (u'CAD', 'C $',  u'Kanadischer Dollar'),
    (u'AUD', 'AU $', u'Australischer Dollar'),
    (u'HKD', 'HK$',  u'Hongkong Dollar'),
    (u'XAG', None, u'Silber'),
    (u'XAU', None, u'Gold'),
    (u'XPD', None, u'Palladium'),
    (u'XPT', None, u'Platin'),
#    (u'XTS', None, u'Fr Testzwecke verwendeter Code'),
    (u'XXX', None, u'Transaktionen, bei denen keine Whrung genutzt wird'), # pylint: disable=W0511
    ]


## \brief List the most known indices
#
#  Entries:
#  \li Name
#  \li Symbol
#  \li ISIN
#  \li WKN
indices_list = [
    (u'DAX (Performance-Index)', u'DAX',  u'DE0008469008', u'846900'),
    (u'DAX (Kurs-Index)',        None,    u'DE0008467440', u'846744'),
    (u'DJ Euro Stoxx 50',        u'SX5E', u'EU0009658145', None),
    (u'MDAX',                    u'MDAX', u'DE0008467416', u'846741'),
    (u'SDAX',                    u'SDXP', u'DE0009653386', u'965338'),
    (u'TecDax',                  u'TDXP', u'DE0007203275', u'720327'),
    ]


## \brief List of some wellknown (german) shares
#
#  Entries:
#  \li Name
#  \li Symbol
#  \li ISIN
#  \li WKN
shares_list = [
    (u'adidas AG',             u'ADS',  u'DE0005003404', u'500340'),
    (u'Allianz SE',            u'ALV',  u'DE0008404005', u'840400'),
    (u'BASF AG',               u'BAS',  u'DE0005151005', u'515100'),
    (u'Bayer AG',              u'BAY',  u'DE0005752000', u'575200'),
    (u'BMW AG St',             u'BMW',  u'DE0005190003', u'519000'),
    (u'Commerzbank AG',        u'CBK',  u'DE0008032004', u'803200'),
    (u'Continental AG',        u'CON',  u'DE0005439004', u'543900'),
    (u'Daimler AG',            u'DAI',  u'DE0007100000', u'710000'),
    (u'Deutsche Bank AG',      u'DBK',  u'DE0005140008', u'514000'),
    (u'Deutsche Boerse AG',    u'DB1',  u'DE0005810055', u'581005'),
    (u'Deutsche Lufthansa AG', u'LHA',  u'DE0008232125', u'823212'),
    (u'Deutsche Post AG',      u'DPW',  u'DE0005552004', u'555200'),
    (u'Deutsche Postbank AG',  u'DPB',  u'DE0008001009', u'800100'),
    (u'Deutsche Telekom AG',   u'DTE',  u'DE0005557508', u'555750'),
    (u'E.ON AG ',              u'EOA',  u'DE0007614406', u'761440'),
    (u'Fresenius Med Care',    u'FME',  u'DE0005785802', u'578580'),
    (u'Henkel KGaA Vz',        u'HEN3', u'DE0006048432', u'604843'),
    (u'Hypo Real Estate Holding AG',  u'HRX',  u'DE0008027707', u'802770'),
    (u'Infineon Technologies AG',     u'IFX',  u'DE0006231004', u'623100'),
    (u'Linde AG',              u'LIN',  u'DE0006483001', u'648300'),
    (u'MAN AG St',             u'MAN',  u'DE0005937007', u'593700'),
    (u'Merck KGaA',            u'MRK',  u'DE0006599905', u'659990'),
    (u'METRO AG St',           u'MEO',  u'DE0007257503', u'725750'),
    (u'Mnchener Rck AG',     u'MUV2', u'DE0008430026', u'843002'),
    (u'RWE AG St',             u'RWE',  u'DE0007037129', u'703712'),
    (u'SAP AG',                u'SAP',  u'DE0007164600', u'716460'),
    (u'Siemens AG',            u'SIE',  u'DE0007236101', u'723610'),
    (u'ThyssenKrupp AG',       u'TKA',  u'DE0007500001', u'750000'),
    (u'TUI AG',                u'TUI1', u'DE000TUAG000', u'TUAG00'),
    (u'Volkswagen AG St',      u'VOW',  u'DE0007664005', u'766400'),
    ]


## \brief Loads the initial dataset into the %database
#
#  \return True on success
#  \see smm.objects.Exchange, smm.objects.Currency, smm.objects.Share
#       smm.objects.DailyQuote, exchange_list and indices_list
def load_initial_data():
    # load som important exchanges
    for symbol in exchange_list:
        ex = objects.Exchange(
            symbol=symbol,
            description=exchange_list[symbol]
            )
        database.store.add(ex)
    database.store.commit()

    # load some important currencies
    for code, sign, description in currencies_list:
        currency = objects.Currency(
            code=code,
            sign=sign,
            description=description
            )
        database.store.add(currency)
    database.store.commit()

    # load some important indices
    for name, symbol, isin, wkn in indices_list:
        share = objects.Share(
            name=name,
            symbol=symbol,
            isin=isin,
            wkn=wkn,
            kind='index',
            )
        database.store.add(share)
    database.store.commit()

    # load some shares
    ex = objects.Exchange.get(symbol='DE')
    for name, symbol, isin, wkn in shares_list:
        share = objects.Share(
            name=name,
            symbol=symbol,
            isin=isin,
            wkn=wkn,
            exchange=ex,
            kind='share'
            )
        database.store.add(share)
    database.store.commit()


## \brief Creates all needed tables to store the %objects inside
#
#  \see smm.objects.Exchange, smm.objects.Currency, smm.objects.Share
#       smm.objects.DailyQuote
#  \note Function doesn't catch any exception
def create_structure():
    # create table to store Exchange objects
    database.store.execute("DROP TABLE IF EXISTS t_exchange;")
    database.store.execute(
        "CREATE TABLE t_exchange ( "
        "id INTEGER PRIMARY KEY AUTOINCREMENT,  "
        "symbol VARCHAR(3) NOT NULL,  "
        "description TEXT NOT NULL "
        ");"
        )
    database.store.execute("DROP INDEX IF EXISTS i_exchange_id;")
    database.store.execute("DROP INDEX IF EXISTS i_exchange_symbol;")
    database.store.execute("CREATE UNIQUE INDEX i_exchange_id ON t_exchange (id);")
    database.store.execute("CREATE UNIQUE INDEX i_exchange_symbol ON t_exchange (symbol);")

    # create table to database.store Currency objects
    database.store.execute("DROP TABLE IF EXISTS t_currency;")
    database.store.execute(
        "CREATE TABLE t_currency ( "
        "id INTEGER PRIMARY KEY AUTOINCREMENT,  "
        "code VARCHAR(3) NOT NULL,  "
        "sign VARCHAR(3),  "
        "description TEXT NOT NULL "
        ");"
        )
    database.store.execute("DROP INDEX IF EXISTS i_currency_id;")
    database.store.execute("DROP INDEX IF EXISTS i_currency_code;")
    database.store.execute("CREATE UNIQUE INDEX i_currency_id ON t_currency (id);")
    database.store.execute("CREATE UNIQUE INDEX i_currency_code ON t_currency (code);")

    # create table to store Share objects
    database.store.execute("DROP TABLE IF EXISTS t_share;")
    database.store.execute(
        "CREATE TABLE t_share ( "
        "id INTEGER PRIMARY KEY AUTOINCREMENT, "
        "symbol VARCHAR(10), "
        "isin VARCHAR(12), "
        "wkn VARCHAR(6), "
        "name TEXT NOT NULL, "
        "currency_id INTEGER, "
        "exchange_id INTEGER, "
        "watch_id INTEGER, "
        "kind INTEGER, "
        "source INTEGER, "
        "notes TEXT "
        ");"
        )
    database.store.execute("DROP INDEX IF EXISTS i_share_id;")
    database.store.execute("DROP INDEX IF EXISTS i_share_isin;")
    database.store.execute("DROP INDEX IF EXISTS i_share_kind;")
    database.store.execute("DROP INDEX IF EXISTS i_share_symbol;")
    database.store.execute("DROP INDEX IF EXISTS i_share_watch_id;")
    database.store.execute("DROP INDEX IF EXISTS i_share_wkn;")
    database.store.execute("CREATE UNIQUE INDEX i_share_id ON t_share (id);")
    database.store.execute("CREATE INDEX i_share_kind ON t_share (kind);")
    database.store.execute("CREATE UNIQUE INDEX i_share_isin ON t_share (isin);")
    database.store.execute("CREATE UNIQUE INDEX i_share_symbol ON t_share (symbol);")
    database.store.execute("CREATE UNIQUE INDEX i_share_watch_id ON t_share (watch_id);")
    database.store.execute("CREATE UNIQUE INDEX i_share_wkn ON t_share (wkn);")

    # create table to store DailyQoute objects
    database.store.execute("DROP TABLE IF EXISTS t_dailyquote;")
    database.store.execute(
        "CREATE TABLE t_dailyquote ( "
        "id INTEGER PRIMARY KEY AUTOINCREMENT, "
        "share_id INTEGER NOT NULL, "
        "exchange_id INTEGER, "
        "currency_id INTEGER, "
        "open DECIMAL, "
        "close DECIMAL, "
        "high DECIMAL, "
        "low DECIMAL, "
        "date DATE NOT NULL, "
        "volume INT, "
        "source INT, "
        "event VARCHAR(20) "
        ");"
        )
    database.store.execute("DROP INDEX IF EXISTS i_dailyquote_id;")
    database.store.execute("DROP INDEX IF EXISTS i_dailyquote_currency_id;")
    database.store.execute("DROP INDEX IF EXISTS i_dailyquote_date;")
    database.store.execute("DROP INDEX IF EXISTS i_dailyquote_share_id;")
    database.store.execute("CREATE UNIQUE INDEX i_dailyquote_id ON t_dailyquote (id);")
    database.store.execute("CREATE INDEX i_dailyquote_currency_id ON t_dailyquote (currency_id);")
    database.store.execute("CREATE INDEX i_dailyquote_date ON t_dailyquote (date);")
    database.store.execute("CREATE INDEX i_dailyquote_share_id ON t_dailyquote (share_id);")

    # create table to store Transaction objects
    database.store.execute("DROP TABLE IF EXISTS t_transaction;")
    database.store.execute(
        "CREATE TABLE t_transaction ( "
        "id INTEGER PRIMARY KEY AUTOINCREMENT, "
        "count DECIMAL NOT NULL, "
        "value DECIMAL, "
        "fee DECIMAL, "
        "date DATE NOT NULL, "
        "exchange_id INTEGER, "
        "kind INTEGER NOT NULL, "
        "position_id INTEGER NOT NULL"
        ");"
        )
    database.store.execute("DROP INDEX IF EXISTS i_transaction_id;")
    database.store.execute("DROP INDEX IF EXISTS i_transaction_position_id;")
    database.store.execute("CREATE UNIQUE INDEX i_transaction_id ON t_transaction (id);")
    database.store.execute("CREATE INDEX i_transaction_position_id ON t_transaction (position_id);")

    # create table to store Watch objects
    database.store.execute("DROP TABLE IF EXISTS t_watch;")
    database.store.execute(
        "CREATE TABLE t_watch ( "
        "id INTEGER PRIMARY KEY AUTOINCREMENT, "
        "created DATE NOT NULL, "
        "expire DATE, "
        "notes TEXT "
        ");"
        )
    database.store.execute("DROP INDEX IF EXISTS i_watch_id;")
    database.store.execute("DROP INDEX IF EXISTS i_watch_expire;")
    database.store.execute("CREATE UNIQUE INDEX i_watch_id ON t_watch (id);")
    database.store.execute("CREATE INDEX i_watch_expire ON t_watch (expire);")

    # create table to store Position objects
    database.store.execute("DROP TABLE IF EXISTS t_position;")
    database.store.execute(
        "CREATE TABLE t_position ( "
        "id INTEGER PRIMARY KEY AUTOINCREMENT, "
        "currency_id INTEGER NOT NULL, "
        "notes TEXT, "
        "open INTEGER NOT NULL, "
        "share_id INTEGER NOT NULL, "
        "stop_id INTEGER "
        ");"
        )
    database.store.execute("DROP INDEX IF EXISTS i_position_id;")
    database.store.execute("DROP INDEX IF EXISTS i_position_open;")
    database.store.execute("CREATE UNIQUE INDEX i_position_id ON t_position (id);")
    database.store.execute("CREATE INDEX i_position_open ON t_position (open);")
