# -*- coding: Latin-1 -*-
#
# SMM Share and Money Management
# Copyright (c) 2007-2010 Carsten Grohmann
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 2 of the
# License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

## \file smm/quote/onvista.py
#
# Functionality to query the finance %database of OnVista.
#
# \namespace smm::quote::onvista
# Functionality to query the finance %database of OnVista.
#
# \author Carsten Grohmann <mail (at) carstengrohmann (dot) de>
#
# \par License:
# GPL version 2.0

import re
import urlparse

from base import FetchPluginBase

## Share data tracker querying quotes using OnVista service
class OnvistaFetchPlugin(FetchPluginBase):

    _specific_properties   = ['currency.code', 'exchange.name', 'share.isin',
                      'share.name', 'quote.nchange', 'quote.pchange', 'quote.price', 'share.wkn', ]

    ## Version number of the interface to access and convert the data
    interface_version = 2

    ## Name of the instance logger
    logger_name = 'smm.quote.onvista'

    ## Name of the data source
    name = u'OnVista'

    ## Unique provider ID
    provider_id = 400

    ## Decimal separator for converting strings to decimal values
    #  \see attr2decimal()
    #  \see tsep
    dsep = u','

    ## Thousands separator for converting strings to decimal values
    #  \see attr2decimal()
    #  \see dsep
    tsep = u'.'

    ## Minimum set of attributes to be extract from the OnVista webpage
    mandatory_attributes = set(['share.name', 'share.wkn', 'share.isin', 'quote.datetime', 'quote.price', \
                                'currency.code', ])

    ## List of URLs to search a share
    #
    #  Element order: general (first) to most special (last).
    #
    #  Be aware of derivated products because the search for products
    #  could return derivated products to.
    #
    #  \see searchURL()
    search_urls = [
        u'http://www.onvista.de/suche.html?SEARCH_VALUE=%(key)s&SELECTED_TOOL=ALL_TOOLS',
        u'http://fonds.onvista.de/suche.html?SEARCH_VALUE=%(key)s',
        ]

    ## Pre-compiled RE to extract date and time of a quote
    REC_DateTime = re.compile(
        r'.*'
        r'(?P<date>[0-9]{2}\.[0-9]{2}\.[0-9]{4}'
        r',\s?'
        r'[0-9]{2}:[0-9]{2}:[0-9]{2}'
        r')'
        r'.*'
        )

    ## Pre-compiled RE to find the text "ISIN:"
    REC_ISINText = re.compile(
        r'^\s*ISIN:\s*$'
        )

    ## Pre-compiled RE to find the product name
    REC_ProductName = re.compile(
        r'^\s*Produktname:\s*$'
        )

    ## Pre-compiled RE to extract the current value, the net change and
    #  the percentage change
    REC_VALUES = re.compile(
        r'^'
        r'(?P<value>[0-9]+,[0-9]+)'                  # value
        r'\s+'
        r'\('
        r'(?P<nchange>[-+]?([0-9]+,[0-9]+|n\.a\.))'  # net change
        r'\s?\/\s?'
        r'(?P<pchange>[-+]?([0-9]+,[0-9]+%|n\.a\.))'  # net change
        r'\)$',
        re.VERBOSE
        )

    ## Pre-compiled RE to find the text "WKN:"
    REC_WKNText = re.compile(
        r'^\s*WKN:\s*$'
        )

    ## Pre-compiled RE to find amount and currency
    REC_AMOUNT = re.compile(
        r'^'
        r'\s*'                              # optional leading spaces
        r'(?P<value>[0-9\.]+,[0-9]+)'       # value e.g. 1.1223,03
        r'\s+'                              # mandatory spaces
        r'(?P<currency>[A-Z]+)'             # currency code
        r'\s*$'                             # optional tailing spaces
        )
    
    ## Obtain information from the data provider
    #
    #  \param self     The object instance itself
    #  \param request  See example
    #
    #  \par Example:
    #  \code
    #  >>> import pprint
    #  >>> pprint.pprint(request)
    #  {u'LU0269579586': {'currency.code': u'EUR',
    #                     'share.isin': u'LU0269579586',
    #                     'share.wkn': u'A0LBQE'}}
    #  >>> provider = smm.quote.onvista.OnvistaFetchPlugin()
    #  >>> result = provider.getCurrent(request)
    #  >>> pprint.pprint(result)
    #  {u'LU0269579586': {'currency.code': u'EUR',
    #                     'errormsg': None,
    #                     'kind': u'fond',
    #                     'quote.datetime': datetime.datetime(2010, 5, 12, 10, 0),
    #                     'quote.nchange': Decimal("0.41"),
    #                     'quote.pchange': Decimal("0.0074"),
    #                     'quote.price': Decimal("55.74"),
    #                     'share.isin': u'LU0269579586',
    #                     'share.name': u'MERIDIO FUNDS - MERIDIO ARAB WORLD P',
    #                     'share.wkn': u'A0LBQE',
    #                     'source.id': 400,
    #                     'source.name': u'OnVista',
    #                     'success': True}}
    #  \endcode
    def getCurrent(self, request):
        # unitialise internal variables
        result = {}

        for key in request:
            isin     = request[key].get('share.isin', None)
            wkn      = request[key].get('share.wkn', None)
            currency = request[key].get('currency.code', None)
            if isin:
                pass
            elif wkn:
                self.logger.warning(
                    _(u'Die Abfrage bei OnVista sollte mit einer ISIN erfolgen. Sie haben nur eine WKN angegeben.'),
                    )
                isin = wkn
            else:
                self.logger.error(
                    _(u'Fr diese Abfrage wurde weder ISIN noch WKN angeben. Es wird daher keine Abfrage bei OnVista durchgefhrt.'),
                    )
                continue

            # compose URL
            url = self.searchURL(isin)
            if not url:
                self.logger.info(
                    _(u'Keine passende Seite bei OnVista fr das Wertpapier "%s" gefunden'),
                    isin,
                    )
                continue

            kind = self.getKind(url)
            if not kind or kind == u'search': 
                self.logger.info(
                    _(u'Keine passende Seite bei OnVista fr das Wertpapier "%s" gefunden'),
                    isin,
                    )
                continue

            # initialise loop internal variables (per ISIN)
            result[key] = self.initShareStruct()
            stock = result[key]
            stock['kind'] = kind
            
            # query data
            conn, errormsg = self.getConnection(url)
            if not conn:
                stock['success'] = False
                stock['errormsg'] = errormsg
                continue

            # fetch the print view
            url = '%s&PRINT=1' % conn.geturl()
            html, errormsg = self.getRemotePage(url)
            if not html:
                self.logger.warning(
                    _(u'Es konnte die Druckansicht "%s" nicht geladen werden'),
                    url,
                    )
                stock['success'] = False
                stock['errormsg'] = errormsg
                continue
        
            # create a HTML soup
            soup = self.getSoup(html)

            # Get WKN and/or ISIN
            ov_wkn = soup.find(text=self.REC_WKNText)
            if ov_wkn:
                stock['share.wkn'] = self.getNextTagText(ov_wkn, trim=True)
               
            ov_isin = soup.find(text=self.REC_ISINText)
            if ov_isin:
                stock['share.isin'] = self.getNextTagText(ov_isin, trim=True)

            # Check got WKN / ISIN against given one
            if stock.get('share.wkn', None) != isin and stock.get('share.isin', None) != isin:
                self.logger.debug(
                    _(u'Es wurde keine WKN / ISIN fr das Wertpapier "%(isin)s" unter "%(url)s" gefunden'),
                    {
                        'isin': isin,
                        'url':  url,
                    }
                    )
                continue
    
            if kind == u'fond':
                self._extractFondDetails(stock, soup, currency)
            elif kind == u'certificate':
                self._extractCertificateDetails(stock, soup)
            else:
                self.logger.warning(
                    _(u'Das Extrahieren von Daten fr "%(isin)s" (Typ: %(kind)s) ist nicht implementiert.'),
                    {
                        'isin': isin,
                        'kind': kind,
                    }
                    )
                continue


            # check for a complete dataset
            current_attributes = set(stock.keys())
            missing_attributes = \
                self.mandatory_attributes.difference(current_attributes)
            if missing_attributes:
                stock['success']  = False
                self.logger.debug(
                    _(u'Folgende Pflichtattribute fehlen: %s'),
                   missing_attributes, 
                    )
                self.logger.debug(
                    _(u'Aktuelle Attribute: %s'),
                    stock
                    )
                if currency:
                    _isin = '%s (%s)' % (isin, currency)
                else:
                    _isin = isin
                stock['errormsg'] = \
                    u'Es wurden keine Daten fr %s zurckgeliefert.' % _isin

        return result


    ## Extract details of fonds from the given soup
    #
    #  \see getCurrent()
    def _extractFondDetails(self, stock, soup, currency):
        # create shortcuts
        tables = soup('table')
        head = tables[4]('tr')

        for pos in [5, 6, 7, 8, 9]:
            # useable data contains the text "Ausgabeaufschlag"
            if not tables[pos].find(text=u'Ausgabeaufschlag'):
                self.logger.debug(
                    _(u'Text "Ausgabeaufschlag" nicht in der Tabelle #%d gefunden. Tabelle wird bersprungen.'),
                    pos,
                    )
                continue
                
            quotes = tables[pos]('tr')
            data = {}
    
            # extract master data
            data['share.name']     = head[0]('td')[0].string
        
            # extract quotes
            amount_text = quotes[2]('td')[0].contents[1].string
            amount_match = self.REC_AMOUNT.search(amount_text)
            if not amount_match:
                self.logger.warning(
                    _(u'Regulrer Ausdruck "REC_AMOUNT" pat nicht auf "%s"'),
                    amount_text,
                    )
                raise NotImplementedError, 'Internal Error'
            else:
                self.attr2decimal(data, 'quote.price', amount_match.group('value'))
                data['currency.code'] = amount_match.group('currency').strip()
            self.attr2decimal(data, 'quote.nchange', quotes[3]('td')[1].contents[0])
            self.attr2decimal(data, 'quote.pchange', quotes[3]('td')[0].contents[0])

            # extract time
            time_text  = quotes[0]('td')[0].contents[2]
            time_match = self.REC_DateTime.search(time_text)
            if not time_match:
                self.logger.debug(
                    _(u'Regulrer Ausdruck "REC_DateTime" pat nicht auf "%s"'),
                    time_text,
                    )
            else:
                self.timestamp2datetime(
                    data,
                    time_match.group('date'),
                    '%d.%m.%Y, %H:%M:%S'
                    )

            if not currency or data['currency.code'] == currency:
                stock.update(data)
                break


    ## Extract details of certificates from the given soup
    #
    #  \see getCurrent()
    def _extractCertificateDetails(self, stock, soup):

        issuer_quotes = soup.find(text='Emittentenkurs:')
        if not issuer_quotes:
            self.logger.warning(
                _(u'Konnte keinen Emittentenkurs finden'),
                )
            stock['success'] = False
            stock['errormsg'] = 'Konnte keinen Emittentenkurs finden'
            return

        timestamp = self.getNextTagText(issuer_quotes)

        # get ask quotes
        ask = soup.find(text=u'Geld (in EUR):')
        ask_text = self.getNextTagText(ask)
        ask_match = self.REC_VALUES.search(ask_text)
        if not ask_match:
            self.logger.debug(
                _(u'Regulrer Ausdruck "REC_VALUES" pat nicht auf "%s"'),
                ask_text,
                )
        else:
            self.attr2decimal(stock, 'quote.ask', ask_match.group('value'))

        # get bid quotes
        bid = soup.find(text=u'Brief (in EUR):')
        bid_text = self.getNextTagText(bid)
        bid_match = self.REC_VALUES.search(bid_text)
        if not bid_match:
            self.logger.debug(
                _(u'Regulrer Ausdruck "REC_VALUES" pat nicht auf "%s"'),
                bid_text,
                )
        else:
            self.attr2decimal(stock, 'quote.bid', bid_match.group('value'))

        # set currency hard coded
        stock['currency.code'] = 'EUR'

        # set last price explicitly
        if 'quote.ask' in stock and 'quote.bid' in stock:
            if stock['quote.ask'] < stock['quote.bid']:
                stock['quote.price'] = stock['quote.ask']
                match = ask_match
            else:
                stock['quote.price'] = stock['quote.bid']
                match = bid_match
            if match.group('nchange'):
                self.attr2decimal(stock, 'quote.nchange', match.group('nchange'))
            if match.group('pchange'):
                self.attr2decimal(stock, 'quote.pchange', match.group('pchange'))
        elif 'quote.ask' in stock and 'quote.bid' not in stock:
            stock['quote.price'] = stock['quote.ask']
        elif 'quote.ask' not in stock and 'quote.bid' in stock:
            stock['quote.price'] = stock['quote.bid']
        else:
            stock['quote.price'] = None

        # OnVista doesn't provide certificate names always
        name = soup.find(text=self.REC_ProductName)
        if name:
            stock['share.name'] = self.getNextTagText(name)
        else:
            stock['share.name'] = None

        self.timestamp2datetime(
            stock, 
            timestamp,
            '(%d.%m., %H:%M:%S)',
            )


    ## Query OnVista for a detail page
    #
    #  \param self The object instance itself
    #  \param key  ISIN or WKN to search (Unicode)
    #
    #  \return     The found URL or \c None on error
    #
    #  \par Example:
    #  \code
    #  >>> provider = smm.OnvistaFetchPlugin()
    #  >>> provider.searchURL('AT0000668264')
    #  'http://fonds.onvista.de/snapshot.html?ID_INSTRUMENT=7347385&SEARCH_VALUE=AT0000668264'
    #  \endcode
    #
    #  \see search_urls and getKind() 
    def searchURL(self, key):
        self.logger.debug(
            _(u'Suche URL fr "%s"'),
            key,
            )

        for pattern in self.search_urls:
            search_url = pattern % {'key': key}
            conn, errormsg = self.getConnection(search_url)
            if errormsg:
                self.logger.error(
                    _(u'Die Suche nach Details fr %(key)s schlug fehl: %(msg)s'),
                    {
                        'key': key,
                        'msg': errormsg,
                    }
                    )
            data_url = conn.geturl()
            kind = self.getKind(data_url)
            if kind not in [None, u'search']:
                self.logger.debug(
                    _(u'URL gefunden: "%s" gefunden.'),
                    data_url
                    )
                return data_url

        # no proper URL found
        self.logger.warning(
            _(u'Keine passende URL bei OnVista fr "%s" gefunden'),
            key
            )
        return None


    ## Determinate the share kind from the given \c url.
    #
    #  \param self  The object instance itself
    #  \param url   URL
    #
    #  \return String with the determinated kind similar to Share.kind or
    #          \c search for URLs with search forms or \c None for unknown
    #          URLs.
    #
    #  \par Example:
    #  \code
    #  >>> provider = smm.OnvistaFetchPlugin()
    #  >>> provider.getKind(u'http://fonds.onvista.de/snapshot.html?ID_INSTRUMENT=7347385&SEARCH_VALUE=AT0000668264')
    #  u'fond'
    #  \endcode
    #
    #  \see Share.kind and searchURL()
    def getKind(self, url):
        # parse url to check if onvista understand our search request
        parts = urlparse.urlparse(url)

        # search page means that the share is unknown or amviguously
        if parts.path.startswith(u'/suche.html') or \
           parts.path.startswith(u'/filter-suche/'):
            return u'search'

        # return
        if parts.hostname == 'aktien.onvista.de':
            res = u'share'
        elif parts.hostname == 'anleihen.onvista.de':
            res = u'bond'
        elif parts.hostname == 'etf.onvista.de':
            res = u'etf'
        elif parts.hostname == 'fonds.onvista.de':
            res = u'fond'
        elif parts.hostname == 'optionsscheine.onvista.de':
            res = u'warrant'
        elif parts.hostname == 'www.onvista.de':
            res = None
        elif parts.hostname == 'zertifikate.onvista.de':
            res = u'certificate'
        else:
            self.logger.warning(
                _(u'OnVista lieferte einen unbekannten Host zurck: %s'),
                parts.hostname
                )
            res = None

        return res
