# -*- coding: Latin-1 -*-
#
# SMM Share and Money Management
# Copyright (c) 2007-2010 Carsten Grohmann
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 2 of the
# License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

## \file smm/quote/vwd.py
#
# Functionality to query the finance database of VWD AG
#
# \namespace smm::quote::vwd
# Functionality to query the finance database of VWD AG
#
# \author Carsten Grohmann <mail (at) carstengrohmann (dot) de>
#
# \par License:
# GPL version 2.0

from base import FetchPluginBase

## Share data tracker querying finance data provided by VWD AG
#
#  \note The website has been changed. Thereby the class doesn't work
#  currently.
class VwdFetchPlugin(FetchPluginBase):

    _specific_properties   = ['quote.ask', 'quote.bid', 'currency.code',
                              'quote.datetime', 'exchange.name', 'share.isin',
                              'share.name', 'quote.nchange', 'quote.pchange',
                              'quote.price', 'share.symbol', 'share.wkn', ]

    ## Version number of the interface to access and convert the data
    interface_version = 2

    ## Name of the instance logger
    logger_name = 'smm.quote.vwd'

    ## Name of the data source
    name = u'vwd'

    ## Unique provider ID
    provider_id = 200

    ## Decimal separator for converting strings to decimal values
    #  \see attr2decimal()
    #  \see tsep
    dsep = u','

    ## Thousands separator for converting strings to decimal values
    #  \see attr2decimal()
    #  \see dsep
    tsep = None

    ## URL base to query the data
    provider_url_base      = 'http://www.finanztreff.de'
    
    ## Reqest to get the first data page
    provider_url_request_1 = 'ftreff/kurse_einzelkurs_uebersicht.htm?s=%(wkn)s'
    
    ## Reqest to get another data page
    provider_url_request_2 = 'ftreff/redirect.htm?kurs=%(wkn)s&fonds=&zertifikate=&os=&turbos=&u=0&p=0&k=0&suche=1'

    ## Obtain information from the data provider
    #
    #  \param self     The object instance itself
    #  \param request  See example
    #
    #  \par Example:
    #  \code
    #  >>> import pprint
    #  >>> pprint.pprint(request)
    #  {u'DE0005003404': {'exchange.symbol': u'DE',
    #                     'share.isin': u'DE0005003404',
    #                     'share.symbol': u'ADS',
    #                     'share.wkn': u'500340'}}
    #  >>> provider = smm.quote.vwd.VwdFetchPlugin()
    #  >>> result = provider.getCurrent(request)
    #  >>> pprint.pprint(result)
    #  {'500340': {'currency.code': u'EUR',
    #              'quote.datetime': datetime.datetime(2008, 1, 25, 19, 43, 52),
    #              'errormsg': None,
    #              'exchange.name': u'Frankfurt',
    #              'share.isin': u'DE0005003404',
    #              'share.name': u'Adidas',
    #              'quote.nchange': Decimal("-1.00"),
    #              'quote.pchange': Decimal("-0.1"),
    #              'quote.price': 'Decimal("1.21"),
    #              'source.id': 200,
    #              'source.name': u'vwd',
    #              'success': True,
    #              'share.symbol': u'ADS',
    #              'share.wkn': u'500340'}}
    #  \endcode
    def getCurrent(self, request):
        # unitialise internal variables
        info = {}

        # compose URL list
        url_list = []
        url_list.append('%s/%s' % (self.provider_url_base, self.provider_url_request_1))
        url_list.append('%s/%s' % (self.provider_url_base, self.provider_url_request_2))

        for key in request:
            wkn      = request[key].get('share.wkn', None)
            currency = request[key].get('currency.code', None)
            if not wkn:
                self.logger.error(
                    _(u'Fr diese Abfrage wurde keine WKN angegeben. Es wird daher keine Abfrage bei VWD durchgefhrt.'),
                    )
                continue

            # initialise loop internal variables (per WKN)
            info[wkn] = self.initShareStruct()
            stock = info[wkn]
            data = {}

            # check all listed URL's for data
            begin = -1
            for url in url_list:
                # query data
                html, errormsg = self.getRemotePage(url % {'wkn': wkn})
                if not html:
                    stock['success'] = False
                    stock['errormsg'] = errormsg
                    continue
    
                begin = html.find('<!-- Stammdaten -->')
                if begin != -1:
                    break

            if begin == -1:
                stock['success'] = False
                if not stock['errormsg']:
                    stock['errormsg'] = _(u'Es wurden keine Daten zurckgeliefert.')
                continue
        
            end = html.rfind('<!-- /Vergleich -->')
            if end > 0:
                html = html[begin:end]
            else:
                html = html[begin:]
        
            # create a HTML soup
            soup = self.getSoup(html)
            tables = soup('table')
        
            # create shortcuts
            stammdaten  = tables[0]('tr')
            jahreschart = tables[1]('tr')
            kursdaten   = tables[2]('tr')
        
            # extract content of table "Stammdaten"
            data['share.name']    = stammdaten[1]('td')[1].string.strip()
            data['share.wkn']     = stammdaten[3]('td')[1].string.strip()
            data['share.symbol']  = stammdaten[4]('td')[1].string.strip()
            data['share.isin']    = stammdaten[5]('td')[1].string.strip()
            data['currency.code'] = stammdaten[8]('td')[1].string.strip()
        
            # extract content of table "Jahreschart"
            cur_date = jahreschart[0]('td')[1].string
        
            # extract content of table "Kursdaten"
            cur_time              = kursdaten[1]('td')[0].contents[-1]
            self.attr2decimal(data, 'quote.price',   kursdaten[1]('td')[1].contents[-1])
            self.attr2decimal(data, 'quote.ask',     kursdaten[5]('td')[1])
            self.attr2decimal(data, 'quote.bid',     kursdaten[4]('td')[1])
            self.attr2decimal(data, 'quote.nchange', kursdaten[2]('td')[0])
            self.attr2decimal(data, 'quote.pchange', kursdaten[1]('td')[2])
        
            # extract date
            self.timestamp2datetime(
                data,
                u'%s %s' % (cur_date, cur_time),
                u'%d.%m.%Y %H:%M:%S'
                )

            # check currency
            if currency and currency != data['currency.code']:
                continue
              
            # apply data
            stock.update(data)
    
        return info
