# -*- coding: Latin-1 -*-
#
# SMM Share and Money Management
# Copyright (c) 2007-2010 Carsten Grohmann
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 2 of the
# License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

## \file smm/quote/yahoo.py
#
# Functionality to query the Yahoo finance database
#
# \namespace smm::quote::yahoo
# Classes to handle finance data provided by Yahoo.
#
# \author Carsten Grohmann <mail (at) carstengrohmann (dot) de>
#
# \par License:
# GPL version 2.0

# import general python modules
import csv
import re
import types

from base import FetchPluginBase

## \page yahoo_symbols Yahoo Symbols
#
#  \par Part of Yahoo Symbols
#  \code
#  Property    Short  Description               SMM Property   Property Type
#  ==========================================================================
#  ask         a      Ask                       quote.ask       Decimal
#  avg_vol     a2     Average Daily Volume
#  bid         b      Bid                       quote.bid       Decimal
#  cap         j1     Market Capitalisation
#  close       p      Previous Close
#  currency    c4     Currency                  currency.code   Unicode
#  date        d1     Last Trade date
#  day_range   m      Days range
#  div_date    r1     Dividend Pay Day
#  div         d      Divident per Share
#  div_yield   y      Divident Yield
#  eps         e      Earnings per Share
#  ex_div      q      Ex-Divident Date
#  last        l1     Last Trade price          quote.price     Decimal
#  name        n      Name                      share.name      Unicode
#  net         c1     Change                    quote.nchange   Decimal
#  open        o      Open                      quote.open      Decimal
#  p_change    p2     Change in percent         quote.pchange   Decimal
#  pe          r      P/E ratio
#  symbol      s      Symbol                    share.symbol    Unicode
#  time        t1     Last Trade time
#  volume      v      Volume                    quote.volume    Decimal
#  year_range  w      Years range
#  \endcode
#
# \par Further Information
#
# A very detailed documenation of the Yahoo symbols is available at
# http://www.gummy-stuff.org/Yahoo-data.htm
#
# A short Python example is available at
# http://www.goldb.org/ystockquote.html
#
# \par Implementations in Perl
#
# The Perl packages Finance::Quote::Yahoo::Base and
# Finance::Quote::Yahoo::Europe are well coded and very good
# example how to query the Yahoo share data.
#
# Current versions are:
# \li http://search.cpan.org/perldoc?Finance::Quote::Yahoo::Base
# \li http://search.cpan.org/perldoc?Finance::Quote::Yahoo::Europe


## Share data tracker querying Yahoo %finance data
#
#  This tracker uses the Yahoo CSV API.
class YahooCSVFetchPlugin(FetchPluginBase):

    ## Yahoo encoded property names
    #
    #  A property correspondes with the name listed at the same position
    #  in field_names.
    #
    #  \see field_names 
    #  \see \ref yahoo_symbols
    field_symbols = ['s', 'n', 'l1', 'd1', 't1', 'c1', 'p2', 'v', 'b', 'a',
                     'p', 'o', 'm', 'w', 'e', 'r', 'r1', 'd', 'y', 'j1', 'q',
                     'a2', 'c4']

    ## Property names
    #
    #  A symbol correspondes with the symbol listed at the same position
    #  in \ref field_symbols.
    #
    #  \see field_symbols 
    #  \see \ref yahoo_symbols
    field_names   = ['symbol', 'name', 'last', 'date', 'time', 'net',
                     'p_change', 'volume', 'bid', 'ask', 'close', 'open',
                     'day_range', 'year_range', 'eps', 'pe', 'div_date',
                     'div', 'div_yield', 'cap', 'ex_div', 'avg_vol',
                     'currency']

    ## Mapping of Yahoo properties to SMM properties
    yahoo2smm = (
        ('currency',        'currency.code'),
        ('errormsg',        'errormsg'),
        ('exchange.symbol', 'exchange.symbol'),
        ('ask',             'quote.ask'),
        ('bid',             'quote.bid'),
        ('quote.datetime',  'quote.datetime'),
        ('high',            'quote.high'),
        ('low',             'quote.low'),
        ('net',             'quote.nchange'),
        ('open',            'quote.open'),
        ('p_change',        'quote.pchange'),
        ('last',            'quote.price'),
        ('volume',          'quote.volume'),
        ('name',            'share.name'),
        ('symbol',          'share.symbol'),
        ('success',         'success'),
       )    

    ## Decimal values
    #
    #  Those returned values will be converted into decimal.Decimal
    #  values
    #
    #  \note Convert only valued needed by SMM.
    #  \see yahoo2smm
    decimal_properties = ['ask', 'bid', 'high', 'last', 'low', 'net', 'open',
                          'p_change', 'volume']

    ## Unicode values
    #
    #  Those returned values will be converted into Unicode values
    #
    #  \note Convert only valued needed by SMM.
    #  \see yahoo2smm
    unicode_properties = ['currency.code', 'exchange.symbol', 
                          'share.name', 'share.symbol']
    
    ## Version number of the interface to access and convert the data
    interface_version = 2

    ## Name of the instance logger
    logger_name = 'smm.quote.yahoocsv'

    ## Name of the data source
    name = u'Yahoo Finance CSV'

    ## Unique provider ID
    provider_id = 100

    ## URL to query the data
    #  
    #  \note We don't use the european Yahoo server, because we've got
    #        wrong / incomplete data
    provider_url = "http://download.finance.yahoo.com/d/quotes.csv"

    ## Decimal separator for converting strings to decimal values
    #  \see attr2decimal()
    #  \see tsep
    dsep = u','

    ## Thousands separator for converting strings to decimal values
    #  \see attr2decimal()
    #  \see dsep
    tsep = None

    def __init__(self):
        FetchPluginBase.__init__(self)
        self.properties.extend(self.field_names[:])
        self.properties = list(set(self.properties))

    ## Obtain information from the data provider
    #
    #  \param self     The object instance itself
    #  \param request  See example
    #
    #  \par Example:
    #  \code
    #  >>> import pprint
    #  >>> pprint.pprint(request)
    #  {u'DE0005003404': {'exchange.symbol': u'DE',
    #                     'share.isin': u'DE0005003404',
    #                     'share.symbol': u'ADS',
    #                     'share.wkn': u'500340'}}
    #  >>> provider = smm.quote.yahoo.YahooCSVFetchPlugin()
    #  >>> result = provider.getCurrent(request)
    #  >>> pprint.pprint(result)
    #  {u'DE0005003404': {'currency.code': 'EUR',
    #                     'errormsg': None,
    #                     'exchange.symbol': u'DE',
    #                     'quote.ask': Decimal("41.745"),
    #                     'quote.bid': Decimal("41.70"),
    #                     'quote.datetime': datetime.datetime(2010, 5, 14, 11, 35),
    #                     'quote.high': Decimal("43.83"),
    #                     'quote.low': Decimal("41.585"),
    #                     'quote.nchange': Decimal("-2.095"),
    #                     'quote.open': Decimal("43.67"),
    #                     'quote.pchange': Decimal("-0.0477"),
    #                     'quote.price': Decimal("41.82"),
    #                     'quote.volume': Decimal("4260280"),
    #                     'share.name': 'ADIDAS',
    #                     'share.symbol': 'ADS.DE',
    #                     'source.id': 100,
    #                     'source.name': u'Yahoo Finance CSV',
    #                     'success': True}}
    #  \endcode
    def getCurrent(self, request):
        # unitialise internal variables
        result = {}
        symbol_list = []
        entry2key = {}

        for key in request:
            symbol   = request[key].get('share.symbol', None)
            exchange = request[key].get('exchange.symbol', None)
            if not symbol:
                self.logger.warning(
                    _(u'Die Anfrage fr %s enthlt kein Symbol. Daher werden keine Daten bei Yahoo ermittelt.'),
                    self._getName(request[key]),
                    )
                continue

            if exchange:
                entry_u = u'%s.%s' % (symbol, exchange)
            else:
                entry_u = u'%s' % symbol

            entry_a = entry_u.encode('ascii', 'replace')

            if '?' in entry_a:
                self.logger.warning(
                    _(u'Die Anfrage fr %s wird nicht durchgefhrt, da Fehler bei der Umformatierung auftraten.'),
                    self._getName(request[key]),
                    )
                continue

            symbol_list.append(entry_a)
            entry2key[entry_a] = key

        # compose URL
        url = '%s?s=%s&f=%s' % (
            self.provider_url,
            '+'.join(symbol_list),
            ''.join(self.field_symbols)
            )

        # query data
        content, errormsg = self.getRemotePage(url)
        if not content:
            self.logger.error(
                _(u'Konnte die Seite "%(url)s" abfragen. Details: "%(msg)s".'),
                {
                    'msg': errormsg,
                    'url': url.decode('ascii', 'replace'),
                },
                )
            return result

        content = content.strip()
        reader = csv.reader(content.split('\n'))

        # evaluate data
        for row in reader:

            entry_u = row[0].decode('ascii', 'replace')
            if entry_u not in entry2key:
                self.logger.info(
                    _(u'Es wurden Kursdaten fr "%s" geliefert, obwohl diese nicht angefordert wurden.'),
                    entry_u
                    )
                continue

            # initialise loop internal variables (per ISIN)
            key      = entry2key[entry_u]
            data     = self.initShareStruct()
            currency = request[key].get('currency.code', None)

            # map list to dictionary
            for pos in range(len(self.field_symbols)):
                field_name = self.field_names[pos]
                try:
                    data[field_name] = row[pos].strip()
                except IndexError:
                    data[field_name] = None


            # process data
            #=============

            # check success
            # Yahoo returns a lot of N/A for non-existent symbols
            if data['date'] == 'N/A' or data['open'] == 'N/A':
                self.logger.info(
                    _(u'Die Abfrage fr "%s" lieferte keine gltigen Daten.'),
                    entry_u,
                    )
                continue

            # split off the exchange 
            if '.' in entry_u:
                data['share.symbol'], data['exchange.symbol'] = entry_u.split('.', 1)
            else:
                data['share.symbol'] = entry_u

            # get low and high watermark from day range
            if '-' in data['day_range']:
                low, high = data['day_range'].split('-', 1)
                data['low'] = low.strip()
                data['high'] = high.strip()

            self.timestamp2datetime(
                data,
                '%s %s' % (data['date'], data['time']),
                '%m/%d/%Y %I:%M%p',
                )

            # convert to Decimal /Unicode values
            for prop in data.keys():
                # skip non-existing or N/A values
                if data[prop] == 'N/A':
                    continue
                if prop in self.decimal_properties:
                    if type(data[prop]) == types.StringType:
                        try:
                            value = data[prop].decode('ascii')
                        except UnicodeDecodeError:
                            self.logger.warn(
                                _(u'Die Umwandlung des Wertes "%s" in einen Unicode-Wert schlug fehl.'),
                                data[prop].decode('ascii', 'replace'),
                                )
                            continue
                    else:
                        value = data[prop]
                    self.attr2decimal(data, prop, value)
                elif prop in self.unicode_properties:
                    try:
                        value = data[prop].decode('ascii', 'replace')
                    except:
                        value = None
                    if value == None:
                        self.logger.debug(
                            _(u'Konnte das Attribute %s mit dem Wert "%s" nicht in eine Unicodezeichenkette umwandeln'),
                            prop,
                            data[prop].decode('ascii', 'replace'),
                            )
                        del data[prop]
                        continue
                    data[prop] = value

            # Check for requested currency
            if currency and data['currency'] != currency:
                self.logger.info(
                    _(u'Die Abfrage fr "%s" lieferte "%s" als Whrung statt "%s".'),
                    entry_u,
                    data['currency'],
                    currency,
                    )
                continue

            # convert yahoo properties into SMM properties
            smm = {}
            for prop_yahoo, prop_smm in self.yahoo2smm:
                if prop_yahoo not in data:
                    self.logger.debug(
                        _(u'Das Yahoo-Attribute %s fehlt und kann daher nicht bernommen werden'),
                        prop_yahoo
                        )
                    continue
                smm[prop_smm] = data[prop_yahoo]
                
            result[key] = smm

        return result

## Share data tracker querying Yahoo %finance data
#
#  This tracker uses the Yahoo web portal
class YahooWebFetchPlugin(FetchPluginBase):

    ## Decimal values
    #
    #  Those returned values will be converted into decimal.Decimal
    #  values
    #
    #  \note Convert only valued needed by SMM.
    #  \see yahoo2smm
    decimal_properties = ['quote.ask', 'quote.bid', 'quote.close', 'quote.high', 'quote.last', 'quote.low',
                          'quote.nchange', 'quote.open', 'quote.pchange', 'quote.price', 'quote.volume']

    ## Name of the data source
    name = u'Yahoo Finance Web'

    ## Version number of the interface to access and convert the data
    interface_version = 2

    ## Name of the instance logger
    logger_name = 'smm.quote.yahooweb'

    ## Unique provider ID
    provider_id = 500

    ## URL to query the data
    provider_url = u"http://de.finance.yahoo.com/q?s=%(isin)s"

    ## Decimal separator for converting strings to decimal values
    #  \see attr2decimal()
    #  \see tsep
    dsep = u','

    ## Thousands separator for converting strings to decimal values
    #  \see attr2decimal()
    #  \see dsep
    tsep = u'.'


    ## \var REC_WKN
    #  Complex pattern to extract WKN and ISIN

    def __init__(self):
        FetchPluginBase.__init__(self)

        # Example: (Frankfurt: DTE.F WKN 555750 / ISIN DE0005557508)
        self.REC_WKN = re.compile(
            r'%s\.%s\ WKN\ %s.*ISIN\ %s' % (
                self.REP_SYMBOL,
                self.REP_EXCHANGE,
                self.REP_WKN,
                self.REP_ISIN,
                ),
            re.VERBOSE
            )

    ## Obtain information from the data provider
    #
    #  \param self      The object instance itself
    #  \param request  See example
    #
    #  \par Example:
    #  \code
    #  >>> import pprint
    #  >>> pprint.pprint(request)
    #  {u'DE0005003404': {'exchange.symbol': u'DE',
    #                     'share.isin': u'DE0005003404',
    #                     'share.symbol': u'ADS',
    #                     'share.wkn': u'500340'}}
    #  >>> provider = smm.quote.yahoo.YahooWebFetchPlugin()
    #  >>> result = provider.getCurrent(request)
    #  >>> pprint.pprint(result)
    #  {u'DE0005003404': {'currency.sign': u'\u20ac',
    #                     'exchange.symbol': u'DE',
    #                     'quote.ask': Decimal("42.06"),
    #                     'quote.bid': Decimal("42.02"),
    #                     'quote.close': Decimal("42.06"),
    #                     'quote.datetime': datetime.datetime(2010, 5, 18, 17, 35),
    #                     'quote.high': Decimal("42.54"),
    #                     'quote.last': Decimal("42.06"),
    #                     'quote.low': Decimal("41.95"),
    #                     'quote.nchange': Decimal("0.35"),
    #                     'quote.open': Decimal("42.10"),
    #                     'quote.pchange': Decimal("0.84"),
    #                     'quote.price': Decimal("42.06"),
    #                     'quote.volume': Decimal("994121"),
    #                     'share.isin': u'DE0005003404',
    #                     'share.wkn': u'500340',
    #                     'source.id': 500,
    #                     'source.name': u'Yahoo Finance Web',
    #                     'success': True}}
    #  \endcode
    def getCurrent(self, request):
        # unitialise internal variables
        info = {}

        for key in request:
            isin          = request[key].get('share.isin', None)
            wkn           = request[key].get('share.wkn', None)
            currency_sign = request[key].get('currency.sign', None)
            if isin:
                pass
            elif wkn:
                self.logger.warning(
                    _(u'Die Abfrage bei Yahoo sollte mit einer ISIN erfolgen. Sie haben nur eine WKN angegeben.'),
                    )
                isin = wkn
            else:
                self.logger.error(
                    _(u'Fr diese Abfrage wurde weder ISIN noch WKN angeben. Es wird daher keine Abfrage bei Yahoo durchgefhrt.'),
                    )
                continue

            # compose URL
            url = self.provider_url % {'isin': key}

            html = self.getHTML(info, key, url, 'WKN ')
            if not html:
                self.logger.debug(
                    _(u'Die Seite "%(url)s" lieferte keine passenden Daten fr "%(key)s".'),
                    {
                        'key': key,
                        'url': url.decode('ascii', 'replace'),
                    },
                    )
                continue

            # create a HTML soup
            soup = self.getSoup(html)

            # create shortcuts
            stock = self.initShareStruct()
            element = soup.find('div', {'class':"yfi_quote_summary"})
            if not element:
                self.logger.debug(
                    _(u'Konnte das Attribute class="yfi_quote_summary" auf der Seite "%s" nicht finden.'),
                    url.decode('ascii', 'replace'),
                    )
                continue

            match = self.REC_WKN.search(element.text)
            if not match:
                self.logger.debug(
                    _(u'Konnte keine WKN / ISIN auf der Seite "%s" nicht finden.'),
                    url.decode('ascii', 'replace'),
                    )
                self.logger.debug(
                    _('Elementtext: %s'),
                    element.text
                    )
                continue
            mg = match.groupdict()
            if 'ex_symbol' in mg:
                stock['exchange.symbol'] = match.group('ex_symbol')
            if 'wkn' in mg:
                stock['share.wkn'] = match.group('wkn')
            if 'isin' in mg:
                stock['share.isin'] = match.group('isin')

            # extract yahoo id
            yid = soup.find('title').string.split(' ')[0].lower()

            for mark, attr in [
                (u'a00', 'quote.ask'),
                (u'b00', 'quote.bid'),
                (u'g00', 'quote.low'),
                (u'h00', 'quote.high'),
                (u'l10', 'quote.last'),
                (u'v00', 'quote.volume'),
                ]:
                span_id = u'yfs_%s_%s' % (mark, yid)
                stock[attr] = None
                element = soup.find(id=span_id)
                if not element:
                    self.logger.debug(
                        _(u'Das Attribut "%(attr)s" ("%(span_id)s") wurde nicht auf der Seite "%(url)s" gefunden.'),
                        {
                            'attr':    attr,
                            'span_id': span_id,
                            'url':     url.decode('ascii', 'replace'),
                        },
                        )
                    continue
                if element.string:
                    stock[attr] = element.string.decode().strip()

            # convert volume format
            stock['quote.volume'] = stock['quote.volume'].replace(u'.', u'')

            # query remaining elements
            stock['quote.open']  = self.getStringFromHtml(soup, u'Erffnungskurs:')
            temp = soup.find(text=u'Letzter Kurs:').next.next
            stock['currency.sign'] = temp.next.contents[-1].string.strip()

            # query date / time
            span_id = u'yfs_t10_%s' % yid
            moment = soup.find(id=span_id).string.strip()
            if u':' in moment:
                self.timestamp2datetime(stock, moment, u'%H:%M')
            else:
                # Fix inconsistent abbreviated month name
                if moment.endswith(u' Mrz'):
                    moment = u'%s Mr' % moment[:-4]
                self.timestamp2datetime(stock, moment, u'%d %b', loc='de')

            change                 = soup.find(text=u'Ver\xe4nderung:')
            stock['quote.nchange'] = change.next('b')[0].string.decode()
            stock['quote.pchange'] = change.next('b')[1].string.decode().strip()[1:-2]

            # Yahoo doesn't provide a closing quote
            # "Letzt. Schluk:" contains yesterday close
            stock['quote.close'] = stock['quote.last']

            # set price explicitly
            stock['quote.price'] = stock['quote.last']

            # convert unicode strings to decimal values
            for prop in self.decimal_properties:
                if type(stock[prop]) != types.UnicodeType:
                    self.logger.debug(
                        _(u'Das Attribute %s wird ignoriert, denn es hat den falschen Datentyp %s statt %s'),
                        prop,
                        type(stock[prop]),
                        types.UnicodeType,
                        )
                    continue
                self.attr2decimal(stock, prop, stock[prop])

            # check for proper currency
            if currency_sign and stock['currency.sign'] != currency_sign:
                self.logger.warning(
                    _(u'Die Abfrage nach Daten fr %s lieferte die falsche Whrung %s statt %s.'),
                    self._getName(request[key]),
                    stock['currency.sign'],
                    currency_sign
                    )
            else:
                info[key] = stock
  
        return info
